@@ -23,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 1125651 $
+ * @version     $Revision: 1305771 $
  */
 
 
@@ -339,7 +339,7 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc,
 
     /* Check if we only have digits in the string */
     for (x = 0; host[x] != '\0'; x++) {
-        if (!isdigit((int)(host[x])) && host[x] != '.') {
+        if (!jk_isdigit(host[x]) && host[x] != '.') {
             break;
         }
     }
@@ -372,8 +372,9 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc,
             temp_sa = temp_sa->next;
 
         /* if temp_sa is set, we have a valid address otherwise, just return */
-        if (NULL != temp_sa)
+        if (NULL != temp_sa) {
             remote_sa = temp_sa;
+        }
         else {
             JK_TRACE_EXIT(l);
             return JK_FALSE;
@@ -959,7 +960,7 @@ char *jk_dump_sinfo(jk_sock_t sd, char *buf)
             return buf;
         }
     }
-    sprintf(buf, "error=%d", errno);
+    sprintf(buf, "errno=%d", errno);
     return buf;
 }
 
