@@ -22,7 +22,7 @@
  * Author:      Mladen Turk <mturk@apache.org>                             *
  * Author:      Rainer Jung <rjung@apache.org>                             *
  * Based on:                                                               *
- * Version:     $Revision: 1137200 $                                          *
+ * Version:     $Revision: 1343092 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -65,6 +65,7 @@ static const char *lb_method_type[] = {
     JK_LB_METHOD_TEXT_TRAFFIC,
     JK_LB_METHOD_TEXT_BUSYNESS,
     JK_LB_METHOD_TEXT_SESSIONS,
+    JK_LB_METHOD_TEXT_NEXT,
     "unknown",
     NULL
 };
@@ -93,6 +94,7 @@ static const char *lb_first_log_names[] = {
     JK_NOTE_LB_FIRST_NAME,
     JK_NOTE_LB_FIRST_VALUE,
     JK_NOTE_LB_FIRST_ACCESSED,
+    JK_NOTE_LB_FIRST_SESSIONS,
     JK_NOTE_LB_FIRST_READ,
     JK_NOTE_LB_FIRST_TRANSFERRED,
     JK_NOTE_LB_FIRST_ERRORS,
@@ -106,6 +108,7 @@ static const char *lb_last_log_names[] = {
     JK_NOTE_LB_LAST_NAME,
     JK_NOTE_LB_LAST_VALUE,
     JK_NOTE_LB_LAST_ACCESSED,
+    JK_NOTE_LB_LAST_SESSIONS,
     JK_NOTE_LB_LAST_READ,
     JK_NOTE_LB_LAST_TRANSFERRED,
     JK_NOTE_LB_LAST_ERRORS,
@@ -158,12 +161,11 @@ int jk_lb_get_lock_code(const char *v)
 {
     if (!v)
         return JK_LB_LOCK_DEF;
-    else if  (*v == 'o' || *v == 'O' || *v == '0')
+    if  (*v == 'o' || *v == 'O' || *v == '0')
         return JK_LB_LOCK_OPTIMISTIC;
-    else if  (*v == 'p' || *v == 'P' || *v == '1')
+    if  (*v == 'p' || *v == 'P' || *v == '1')
         return JK_LB_LOCK_PESSIMISTIC;
-    else
-        return JK_LB_LOCK_DEF;
+    return JK_LB_LOCK_DEF;
 }
 
 /* Return the string representation of the lb method type */
@@ -177,16 +179,17 @@ int jk_lb_get_method_code(const char *v)
 {
     if (!v)
         return JK_LB_METHOD_DEF;
-    else if  (*v == 'r' || *v == 'R' || *v == '0')
+    if  (*v == 'r' || *v == 'R' || *v == '0')
         return JK_LB_METHOD_REQUESTS;
-    else if  (*v == 't' || *v == 'T' || *v == '1')
+    if  (*v == 't' || *v == 'T' || *v == '1')
         return JK_LB_METHOD_TRAFFIC;
-    else if  (*v == 'b' || *v == 'B' || *v == '2')
+    if  (*v == 'b' || *v == 'B' || *v == '2')
         return JK_LB_METHOD_BUSYNESS;
-    else if  (*v == 's' || *v == 'S' || *v == '3')
+    if  (*v == 's' || *v == 'S' || *v == '3')
         return JK_LB_METHOD_SESSIONS;
-    else
-        return JK_LB_METHOD_DEF;
+    if  (*v == 'n' || *v == 'N' || *v == '4')
+        return JK_LB_METHOD_NEXT;
+    return JK_LB_METHOD_DEF;
 }
 
 /* Return the string representation of the balance worker state */
@@ -200,22 +203,21 @@ int jk_lb_get_state_code(const char *v)
 {
     if (!v)
         return JK_LB_STATE_DEF;
-    else if  (*v == 'i' || *v == 'I' || *v == 'n' || *v == 'N' || *v == '0')
+    if  (*v == 'i' || *v == 'I' || *v == 'n' || *v == 'N' || *v == '0')
         return JK_LB_STATE_IDLE;
-    else if  (*v == 'o' || *v == 'O' || *v == '1')
+    if  (*v == 'o' || *v == 'O' || *v == '1')
         return JK_LB_STATE_OK;
-    else if  (*v == 'r' || *v == 'R' || *v == '2')
+    if  (*v == 'r' || *v == 'R' || *v == '2')
         return JK_LB_STATE_RECOVER;
-    else if  (*v == 'f' || *v == 'F' || *v == '3')
+    if  (*v == 'f' || *v == 'F' || *v == '3')
         return JK_LB_STATE_FORCE;
-    else if  (*v == 'b' || *v == 'B' || *v == '4')
+    if  (*v == 'b' || *v == 'B' || *v == '4')
         return JK_LB_STATE_BUSY;
-    else if  (*v == 'e' || *v == 'E' || *v == '5')
+    if  (*v == 'e' || *v == 'E' || *v == '5')
         return JK_LB_STATE_ERROR;
-    else if  (*v == 'p' || *v == 'P' || *v == '6')
+    if  (*v == 'p' || *v == 'P' || *v == '6')
         return JK_LB_STATE_PROBE;
-    else
-        return JK_LB_STATE_DEF;
+    return JK_LB_STATE_DEF;
 }
 
 /* Return the string representation of the balance worker activation */
@@ -236,14 +238,13 @@ int jk_lb_get_activation_code(const char *v)
 {
     if (!v)
         return JK_LB_ACTIVATION_DEF;
-    else if (*v == 'a' || *v == 'A' || *v == '0')
+    if (*v == 'a' || *v == 'A' || *v == '0')
         return JK_LB_ACTIVATION_ACTIVE;
-    else if (*v == 'd' || *v == 'D' || *v == '1')
+    if (*v == 'd' || *v == 'D' || *v == '1')
         return JK_LB_ACTIVATION_DISABLED;
-    else if (*v == 's' || *v == 'S' || *v == '2')
+    if (*v == 's' || *v == 'S' || *v == '2')
         return JK_LB_ACTIVATION_STOPPED;
-    else
-        return JK_LB_ACTIVATION_DEF;
+    return JK_LB_ACTIVATION_DEF;
 }
 
 /* Update the load multipliers wrt. lb_factor */
@@ -281,6 +282,30 @@ void reset_lb_values(lb_worker_t *p, jk_logger_t *l)
     JK_TRACE_EXIT(l);
 }
 
+static void jk_lb_pull_worker(lb_worker_t *p, int i, jk_logger_t *l)
+{
+    lb_sub_worker_t *w = &p->lb_workers[i];
+    if (w->sequence < w->s->h.sequence) {
+        jk_worker_t *jw = w->worker;
+        ajp_worker_t *aw = (ajp_worker_t *)jw->worker_private;
+
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "syncing mem for member '%s' of lb '%s' from shm",
+                   w->name, p->name);
+
+        jk_ajp_pull(aw, JK_TRUE, l);
+        strncpy(w->route, w->s->route, JK_SHM_STR_SIZ);
+        strncpy(w->domain, w->s->domain, JK_SHM_STR_SIZ);
+        strncpy(w->redirect, w->s->redirect, JK_SHM_STR_SIZ);
+        w->distance = w->s->distance;
+        w->activation = w->s->activation;
+        w->lb_factor = w->s->lb_factor;
+        w->lb_mult = w->s->lb_mult;
+        w->sequence = w->s->h.sequence;
+    }
+}
+
 /* Syncing config values from shm */
 void jk_lb_pull(lb_worker_t *p, int locked, jk_logger_t *l)
 {
@@ -294,6 +319,11 @@ void jk_lb_pull(lb_worker_t *p, int locked, jk_logger_t *l)
                p->name, p->sequence, p->s->h.sequence);
     if (locked == JK_FALSE)
         jk_shm_lock();
+    if (p->sequence == p->s->h.sequence) {
+        if (locked == JK_FALSE)
+            jk_shm_unlock();
+        return;
+    }
     p->sticky_session = p->s->sticky_session;
     p->sticky_session_force = p->s->sticky_session_force;
     p->recover_wait_time = p->s->recover_wait_time;
@@ -304,32 +334,13 @@ void jk_lb_pull(lb_worker_t *p, int locked, jk_logger_t *l)
     p->lbmethod = p->s->lbmethod;
     p->lblock = p->s->lblock;
     p->max_packet_size = p->s->max_packet_size;
-    p->sequence = p->s->h.sequence;
     strncpy(p->session_cookie, p->s->session_cookie, JK_SHM_STR_SIZ);
     strncpy(p->session_path, p->s->session_path, JK_SHM_STR_SIZ);
 
     for (i = 0; i < p->num_of_workers; i++) {
-        lb_sub_worker_t *w = &p->lb_workers[i];
-        if (w->sequence != w->s->h.sequence) {
-            jk_worker_t *jw = w->worker;
-            ajp_worker_t *aw = (ajp_worker_t *)jw->worker_private;
-
-            if (JK_IS_DEBUG_LEVEL(l))
-                jk_log(l, JK_LOG_DEBUG,
-                       "syncing mem for member '%s' of lb '%s' from shm",
-                       w->name, p->name);
-
-            jk_ajp_pull(aw, JK_TRUE, l);
-            strncpy(w->route, w->s->route, JK_SHM_STR_SIZ);
-            strncpy(w->domain, w->s->domain, JK_SHM_STR_SIZ);
-            strncpy(w->redirect, w->s->redirect, JK_SHM_STR_SIZ);
-            w->distance = w->s->distance;
-            w->activation = w->s->activation;
-            w->lb_factor = w->s->lb_factor;
-            w->lb_mult = w->s->lb_mult;
-            w->sequence = w->s->h.sequence;
-        }
+        jk_lb_pull_worker(p, i, l);
     }
+    p->sequence = p->s->h.sequence;
     if (locked == JK_FALSE)
         jk_shm_unlock();
 
@@ -359,7 +370,6 @@ void jk_lb_push(lb_worker_t *p, int locked, jk_logger_t *l)
     p->s->lbmethod = p->lbmethod;
     p->s->lblock = p->lblock;
     p->s->max_packet_size = p->max_packet_size;
-    p->s->h.sequence = p->sequence;
     strncpy(p->s->session_cookie, p->session_cookie, JK_SHM_STR_SIZ);
     strncpy(p->s->session_path, p->session_path, JK_SHM_STR_SIZ);
 
@@ -382,9 +392,16 @@ void jk_lb_push(lb_worker_t *p, int locked, jk_logger_t *l)
             w->s->activation = w->activation;
             w->s->lb_factor = w->lb_factor;
             w->s->lb_mult = w->lb_mult;
-            w->s->h.sequence = w->sequence;
+            w->s->h.sequence++;
+            w->sequence = w->s->h.sequence;
         }
     }
+    /* Increment the shared memory sequence number.
+     * Our number can be behind the actual value in
+     * shared memory.
+     */
+    p->s->h.sequence++;
+    p->sequence = p->s->h.sequence;
     if (locked == JK_FALSE)
         jk_shm_unlock();
 
@@ -441,9 +458,9 @@ static char *get_cookie(jk_ws_service_t *s, const char *name)
                  id_start; id_start = strstr(id_start + 1, name)) {
                 if (id_start == s->headers_values[i] ||
                     id_start[-1] == ';' ||
-                    id_start[-1] == ',' || isspace((int)id_start[-1])) {
+                    id_start[-1] == ',' || jk_isspace(id_start[-1])) {
                     id_start += strlen(name);
-                    while (*id_start && isspace((int)(*id_start)))
+                    while (*id_start && jk_isspace(*id_start))
                         ++id_start;
                     if (*id_start == '=' && id_start[1]) {
                         /*
@@ -487,9 +504,30 @@ static char *get_cookie(jk_ws_service_t *s, const char *name)
 static char *get_sessionid(jk_ws_service_t *s, lb_worker_t *p, jk_logger_t *l)
 {
     char *val;
-    val = get_path_param(s, p->session_path);
+    char *session_path;
+    char *session_cookie;
+
+    /* If the web server sets a route, ignore the real session id
+     * and fake a new one for that route.
+     */
+    if (s->route) {
+        size_t sz = strlen(s->route) + 1;
+        val = jk_pool_alloc(s->pool, sz + 1);
+        val[0] = '.';
+        memcpy(val + 1, s->route, sz);
+        return val;
+    }
+
+    // set session_path
+    session_path = (s->extension.session_path) ?
+            s->extension.session_path : p->session_path;
+    // set session_cookie
+    session_cookie = (s->extension.session_cookie) ?
+            s->extension.session_cookie : p->session_cookie;
+
+    val = get_path_param(s, session_path);
     if (!val) {
-        val = get_cookie(s, p->session_cookie);
+        val = get_cookie(s, session_cookie);
     }
     if (val && !*val) {
         /* TODO: For now only log the empty sessions.
@@ -533,7 +571,7 @@ static int recover_workers(lb_worker_t *p,
     ajp_worker_t *aw = NULL;
     JK_TRACE_ENTER(l);
 
-    if (p->sequence != p->s->h.sequence)
+    if (p->sequence < p->s->h.sequence)
         jk_lb_pull(p, JK_TRUE, l);
     for (i = 0; i < p->num_of_workers; i++) {
         w = &p->lb_workers[i];
@@ -559,7 +597,8 @@ static int recover_workers(lb_worker_t *p,
             }
         }
         else if (w->s->error_time > 0 &&
-                 (int)difftime(now, w->s->error_time) >= p->error_escalation_time) {
+                 (int)difftime(now, w->s->error_time) >= p->error_escalation_time &&
+                 w->s->state != JK_LB_STATE_RECOVER) {
             if (JK_IS_DEBUG_LEVEL(l))
                 jk_log(l, JK_LOG_DEBUG,
                        "worker %s escalating local error to global error",
@@ -619,6 +658,8 @@ static jk_uint64_t decay_load(lb_worker_t *p,
 {
     unsigned int i;
     jk_uint64_t curmax = 0;
+    jk_uint64_t curmin = 0;
+    int no_curmin = JK_TRUE;
     lb_sub_worker_t *w;
     ajp_worker_t *aw;
 
@@ -626,7 +667,9 @@ static jk_uint64_t decay_load(lb_worker_t *p,
     for (i = 0; i < p->num_of_workers; i++) {
         w = &p->lb_workers[i];
         if (p->lbmethod != JK_LB_METHOD_BUSYNESS) {
-            w->s->lb_value >>= exponent;
+            if (p->lbmethod != JK_LB_METHOD_NEXT) {
+                w->s->lb_value >>= exponent;
+            }
             if (w->s->lb_value > curmax) {
                 curmax = w->s->lb_value;
             }
@@ -634,6 +677,29 @@ static jk_uint64_t decay_load(lb_worker_t *p,
         aw = (ajp_worker_t *)w->worker->worker_private;
         aw->s->reply_timeouts >>= exponent;
     }
+    if (p->lbmethod == JK_LB_METHOD_NEXT) {
+        for (i = 0; i < p->num_of_workers; i++) {
+            w = &p->lb_workers[i];
+            /* Take into account only the workers that are
+             * not in error state, stopped, disabled or busy.
+             * Unfortunately we can not respect activations
+             * defined by mapping rules here.
+             */
+            if (JK_WORKER_USABLE(w->s->state, w->activation)) {
+                if (w->s->lb_value < curmin || no_curmin == JK_TRUE) {
+                    no_curmin = JK_FALSE;
+                    curmin = w->s->lb_value;
+                }
+            }
+        }
+        for (i = 0; i < p->num_of_workers; i++) {
+            w = &p->lb_workers[i];
+            if (w->s->lb_value >= curmin)
+                w->s->lb_value -= curmin;
+            else
+                w->s->lb_value = 0;
+        }
+    }
     JK_TRACE_EXIT(l);
     return curmax;
 }
@@ -734,7 +800,7 @@ static int find_best_bydomain(jk_ws_service_t *s,
             strlen(wr.domain) != domain_len ||
             strncmp(wr.domain, route_or_domain, domain_len))
             continue;
-        /* Take into calculation only the workers that are
+        /* Take into account only the workers that are
          * not in error state, stopped, disabled or busy.
          */
         activation = s->extension.activation ?
@@ -784,13 +850,14 @@ static int find_best_byvalue(jk_ws_service_t *s,
         if (activation == JK_LB_ACTIVATION_UNSET)
             activation = wr.activation;
 
-        /* Take into calculation only the workers that are
+        /* Take into account only the workers that are
          * not in error state, stopped, disabled or busy.
          */
         if (JK_WORKER_USABLE(states[wr.i], activation)) {
             if (candidate < 0 || wr.distance < d ||
-                (wr.s->lb_value < curmin &&
-                wr.distance == d)) {
+                (s->extension.stateless != JK_TRUE &&
+                 wr.s->lb_value < curmin &&
+                 wr.distance == d)) {
                 candidate = i;
                 curmin = wr.s->lb_value;
                 d = wr.distance;
@@ -902,7 +969,6 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
                                     jk_logger_t *l)
 {
     int rc = -1;
-    int r;
 
     JK_TRACE_ENTER(l);
     if (p->num_of_workers == 1) {
@@ -925,15 +991,15 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
             return -1;
         }
     }
-    if (p->lblock == JK_LB_LOCK_PESSIMISTIC)
-        r = jk_shm_lock();
-    else {
-        JK_ENTER_CS(&(p->cs), r);
+    if (p->lblock == JK_LB_LOCK_PESSIMISTIC) {
+        if (!jk_shm_lock()) {
+            jk_log(l, JK_LOG_ERROR, "locking failed (errno=%d)", errno);
+            JK_TRACE_EXIT(l);
+            return -1;
+        }
     }
-    if (!r) {
-       jk_log(l, JK_LOG_ERROR,
-              "locking failed (errno=%d)",
-              errno);
+    else {
+        JK_ENTER_CS(&p->cs);
     }
     if (sessionid) {
         char *session = sessionid;
@@ -959,10 +1025,11 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
                 rc = find_bysession_route(s, p, session_route, states, l);
                 if (rc >= 0) {
                     lb_sub_worker_t *wr = &(p->lb_workers[rc]);
-                    if (p->lblock == JK_LB_LOCK_PESSIMISTIC)
+                    if (p->lblock == JK_LB_LOCK_PESSIMISTIC) {
                         jk_shm_unlock();
+                    }
                     else {
-                        JK_LEAVE_CS(&(p->cs), r);
+                        JK_LEAVE_CS(&p->cs);
                     }
                     if (JK_IS_DEBUG_LEVEL(l))
                         jk_log(l, JK_LOG_DEBUG,
@@ -977,10 +1044,11 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
             rc = -1;
         }
         if (rc < 0 && p->sticky_session_force) {
-            if (p->lblock == JK_LB_LOCK_PESSIMISTIC)
+            if (p->lblock == JK_LB_LOCK_PESSIMISTIC) {
                 jk_shm_unlock();
+            }
             else {
-                JK_LEAVE_CS(&(p->cs), r);
+                JK_LEAVE_CS(&p->cs);
             }
             jk_log(l, JK_LOG_INFO,
                    "all workers are in error state for session %s",
@@ -990,10 +1058,11 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
         }
     }
     rc = find_best_worker(s, p, states, l);
-    if (p->lblock == JK_LB_LOCK_PESSIMISTIC)
+    if (p->lblock == JK_LB_LOCK_PESSIMISTIC) {
         jk_shm_unlock();
+    }
     else {
-        JK_LEAVE_CS(&(p->cs), r);
+        JK_LEAVE_CS(&p->cs);
     }
     if (rc >= 0) {
         lb_sub_worker_t *wr = &(p->lb_workers[rc]);
@@ -1027,25 +1096,29 @@ static void lb_add_log_items(jk_ws_service_t *s,
         /* JK_NOTE_LB_FIRST/LAST_ACCESSED */
         log_values[2] = buf;
         buf += JK_LB_UINT64_STR_SZ;
+        snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, w->s->sessions);
+        /* JK_NOTE_LB_FIRST/LAST_SESSIONS */
+        log_values[3] = buf;
+        buf += JK_LB_UINT64_STR_SZ;
         snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, aw->s->readed);
         /* JK_NOTE_LB_FIRST/LAST_READ */
-        log_values[3] = buf;
+        log_values[4] = buf;
         buf += JK_LB_UINT64_STR_SZ;
         snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT64_T_FMT, aw->s->transferred);
         /* JK_NOTE_LB_FIRST/LAST_TRANSFERRED */
-        log_values[4] = buf;
+        log_values[5] = buf;
         buf += JK_LB_UINT64_STR_SZ;
         snprintf(buf, JK_LB_UINT64_STR_SZ, "%" JK_UINT32_T_FMT, w->s->errors);
         /* JK_NOTE_LB_FIRST/LAST_ERRORS */
-        log_values[5] = buf;
+        log_values[6] = buf;
         buf += JK_LB_UINT64_STR_SZ;
         snprintf(buf, JK_LB_UINT64_STR_SZ, "%d", aw->s->busy);
         /* JK_NOTE_LB_FIRST/LAST_BUSY */
-        log_values[6] = buf;
+        log_values[7] = buf;
         /* JK_NOTE_LB_FIRST/LAST_ACTIVATION */
-        log_values[7] = jk_lb_get_activation(w, l);
+        log_values[8] = jk_lb_get_activation(w, l);
         /* JK_NOTE_LB_FIRST/LAST_STATE */
-        log_values[8] = jk_lb_get_state(w, l);
+        log_values[9] = jk_lb_get_state(w, l);
         s->add_log_items(s, log_names, log_values, JK_LB_NOTES_COUNT);
     }
 }
@@ -1082,7 +1155,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
     /* Set returned error to OK */
     *is_error = JK_HTTP_OK;
 
-    if (p->worker->sequence != p->worker->s->h.sequence)
+    if (p->worker->sequence < p->worker->s->h.sequence)
         jk_lb_pull(p->worker, JK_FALSE, l);
     for (i = 0; i < num_of_workers; i++) {
         lb_sub_worker_t *rec = &(p->worker->lb_workers[i]);
@@ -1111,16 +1184,16 @@ static int JK_METHOD service(jk_endpoint_t *e,
     if (jk_b_set_buffer_size(s->reco_buf, p->worker->max_packet_size)) {
         *is_error = JK_HTTP_SERVER_ERROR;
         jk_log(l, JK_LOG_ERROR,
-               "Failed allocating AJP message buffer");
+               "Failed allocating AJP message buffer of %d bytes.", p->worker->max_packet_size);
         JK_TRACE_EXIT(l);
         return JK_SERVER_ERROR;
     }
     jk_b_reset(s->reco_buf);
     s->reco_status = RECO_INITED;
 
-    if (p->worker->sticky_session) {
+    if (p->worker->sticky_session && s->extension.sticky_ignore != JK_TRUE) {
         /* Use sessionid only if sticky_session is
-         * defined for this load balancer
+         * defined and not overwritten for this load balancer
          */
         sessionid = get_sessionid(s, p->worker, l);
     }
@@ -1142,7 +1215,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
                        retry, p->worker->retry_interval);
             jk_sleep(p->worker->retry_interval);
             /* Pull shared memory if something changed during sleep */
-            if (p->worker->sequence != p->worker->s->h.sequence)
+            if (p->worker->sequence < p->worker->s->h.sequence)
                 jk_lb_pull(p->worker, JK_FALSE, l);
             for (i = 0; i < num_of_workers; i++) {
                 /* Copy the shared state info */
@@ -1219,11 +1292,16 @@ static int JK_METHOD service(jk_endpoint_t *e,
                 rec->s->busy++;
                 if (p->worker->s->busy > p->worker->s->max_busy)
                     p->worker->s->max_busy = p->worker->s->busy;
-                if ( (p->worker->lbmethod == JK_LB_METHOD_REQUESTS) ||
-                     (p->worker->lbmethod == JK_LB_METHOD_BUSYNESS) ||
-                     (p->worker->lbmethod == JK_LB_METHOD_SESSIONS &&
-                      !sessionid) )
+                if (p->worker->lbmethod == JK_LB_METHOD_REQUESTS ||
+                    p->worker->lbmethod == JK_LB_METHOD_BUSYNESS ||
+                    (!sessionid &&
+                     s->extension.stateless != JK_TRUE &&
+                     (p->worker->lbmethod == JK_LB_METHOD_SESSIONS ||
+                      p->worker->lbmethod == JK_LB_METHOD_NEXT)))
                     rec->s->lb_value += rec->lb_mult;
+                if (!sessionid && s->extension.stateless != JK_TRUE) {
+                    rec->s->sessions++;
+                }
                 if (p->worker->lblock == JK_LB_LOCK_PESSIMISTIC)
                     jk_shm_unlock();
 
@@ -1523,6 +1601,7 @@ static int JK_METHOD validate(jk_worker_t *pThis,
         unsigned int num_of_workers;
         const char *secret;
 
+        p->worker.we = we;
         p->sticky_session = jk_get_is_sticky_session(props, p->name);
         p->sticky_session_force = jk_get_is_sticky_session_force(props, p->name);
         secret = jk_get_worker_secret(props, p->name);
@@ -1533,7 +1612,6 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                                   &num_of_workers) && num_of_workers) {
             unsigned int i = 0;
             unsigned int j = 0;
-            p->max_packet_size = DEF_BUFFER_SZ;
             p->lb_workers = jk_pool_alloc(&p->p,
                                           num_of_workers *
                                           sizeof(lb_sub_worker_t));
@@ -1543,26 +1621,39 @@ static int JK_METHOD validate(jk_worker_t *pThis,
             }
             memset(p->lb_workers, 0, num_of_workers * sizeof(lb_sub_worker_t));
             for (i = 0; i < num_of_workers; i++) {
-                p->lb_workers[i].s = jk_shm_alloc_lb_sub_worker(&p->p);
+                p->lb_workers[i].s = jk_shm_alloc_lb_sub_worker(&p->p, p->s->h.id, worker_names[i]);
                 if (p->lb_workers[i].s == NULL) {
                     jk_log(l, JK_LOG_ERROR,
                            "allocating lb sub worker record from shared memory");
                     JK_TRACE_EXIT(l);
                     return JK_FALSE;
                 }
+                p->lb_workers[i].i = i;
+                strncpy(p->lb_workers[i].name, worker_names[i], JK_SHM_STR_SIZ);
             }
 
             for (i = 0; i < num_of_workers; i++) {
                 const char *s;
                 unsigned int ms;
 
-                p->lb_workers[i].i = i;
-                strncpy(p->lb_workers[i].name, worker_names[i],
-                        JK_SHM_STR_SIZ);
-                strncpy(p->lb_workers[i].s->h.name, worker_names[i],
-                        JK_SHM_STR_SIZ);
-                p->lb_workers[i].sequence = 0;
-                p->lb_workers[i].s->h.sequence = 0;
+                if (p->lb_workers[i].s->h.sequence != 0) {
+                    /* Somebody already setup this worker.
+                     * Just pull the data.
+                     */
+                    if (JK_IS_DEBUG_LEVEL(l)) {
+                        jk_log(l, JK_LOG_DEBUG,
+                               "Balanced worker %s already configured (sequence=%d)",
+                               p->lb_workers[i].name, p->lb_workers[i].s->h.sequence);
+                    }           
+                    if (!wc_create_worker(p->lb_workers[i].name, 0,
+                                          props,
+                                          &(p->lb_workers[i].worker),
+                                          we, l) || !p->lb_workers[i].worker) {
+                        break;
+                    }
+                    jk_lb_pull_worker(p, i, l);
+                    continue;
+                }
                 p->lb_workers[i].lb_factor =
                     jk_get_lb_factor(props, worker_names[i]);
                 if (p->lb_workers[i].lb_factor < 1) {
@@ -1588,6 +1679,8 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                 p->lb_workers[i].s->lb_value = 0;
                 p->lb_workers[i].s->state = JK_LB_STATE_IDLE;
                 p->lb_workers[i].s->error_time = 0;
+                p->lb_workers[i].s->elected_snapshot = 0;
+                p->lb_workers[i].s->sessions = 0;
                 p->lb_workers[i].activation =
                     jk_get_worker_activation(props, worker_names[i]);
                 if (!wc_create_worker(p->lb_workers[i].name, 0,
@@ -1713,10 +1806,17 @@ static int JK_METHOD init(jk_worker_t *pThis,
         JK_TRACE_EXIT(log);
         return JK_FALSE;
     }
-
-    p->sequence++;
-    jk_lb_push(p, JK_FALSE, log);
-
+    if (p->s->h.sequence == 0) {
+        /* Set configuration data to shared memory
+         */
+        jk_lb_push(p, JK_TRUE, log);
+    }
+    else {
+        /* Shared memory for this worker is already configured.
+         * Update with runtime data
+         */
+        jk_lb_pull(p, JK_TRUE, log);
+    }
     JK_TRACE_EXIT(log);
     return JK_TRUE;
 }
@@ -1757,11 +1857,10 @@ static int JK_METHOD destroy(jk_worker_t **pThis, jk_logger_t *l)
     JK_TRACE_ENTER(l);
 
     if (pThis && *pThis && (*pThis)->worker_private) {
-        unsigned int i;
         lb_worker_t *private_data = (*pThis)->worker_private;
 
         close_workers(private_data, private_data->num_of_workers, l);
-        JK_DELETE_CS(&(private_data->cs), i);
+        JK_DELETE_CS(&private_data->cs);
         jk_close_pool(&private_data->p);
         free(private_data);
 
@@ -1788,14 +1887,13 @@ int JK_METHOD lb_worker_factory(jk_worker_t **w,
                         private_data->buf,
                         sizeof(jk_pool_atom_t) * TINY_POOL_SIZE);
 
-        private_data->s = jk_shm_alloc_lb_worker(&private_data->p);
+        private_data->s = jk_shm_alloc_lb_worker(&private_data->p, name);
         if (!private_data->s) {
             free(private_data);
             JK_TRACE_EXIT(l);
             return 0;
         }
         strncpy(private_data->name, name, JK_SHM_STR_SIZ);
-        strncpy(private_data->s->h.name, name, JK_SHM_STR_SIZ);
         private_data->lb_workers = NULL;
         private_data->num_of_workers = 0;
         private_data->worker.worker_private = private_data;
@@ -1807,8 +1905,8 @@ int JK_METHOD lb_worker_factory(jk_worker_t **w,
         private_data->recover_wait_time = WAIT_BEFORE_RECOVER;
         private_data->error_escalation_time = private_data->recover_wait_time / 2;
         private_data->max_reply_timeouts = 0;
+        private_data->max_packet_size = DEF_BUFFER_SZ;
         private_data->sequence = 0;
-        private_data->s->h.sequence = 0;
         private_data->next_offset = 0;
         *w = &private_data->worker;
         JK_TRACE_EXIT(l);
