@@ -19,7 +19,7 @@
  * Description: load balance worker header file                            *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 751916 $                                           *
+ * Version:     $Revision: 1174411 $                                           *
  ***************************************************************************/
 
 #ifndef JK_LB_WORKER_H
@@ -44,12 +44,14 @@ extern "C"
 #define JK_LB_METHOD_TRAFFIC           (1)
 #define JK_LB_METHOD_BUSYNESS          (2)
 #define JK_LB_METHOD_SESSIONS          (3)
+#define JK_LB_METHOD_NEXT              (4)
 #define JK_LB_METHOD_DEF               (JK_LB_METHOD_REQUESTS)
-#define JK_LB_METHOD_MAX               (JK_LB_METHOD_SESSIONS)
+#define JK_LB_METHOD_MAX               (JK_LB_METHOD_NEXT)
 #define JK_LB_METHOD_TEXT_REQUESTS     ("Request")
 #define JK_LB_METHOD_TEXT_TRAFFIC      ("Traffic")
 #define JK_LB_METHOD_TEXT_BUSYNESS     ("Busyness")
 #define JK_LB_METHOD_TEXT_SESSIONS     ("Sessions")
+#define JK_LB_METHOD_TEXT_NEXT         ("Next")
 #define JK_LB_METHOD_TEXT_DEF          (JK_LB_METHOD_TEXT_REQUESTS)
 #define JK_LB_LOCK_OPTIMISTIC          (0)
 #define JK_LB_LOCK_PESSIMISTIC         (1)
@@ -101,10 +103,11 @@ extern "C"
 #define JK_LB_ACTIVATION_TEXT_DEF      (JK_LB_ACTIVATION_TEXT_ACTIVE)
 
 #define JK_LB_UINT64_STR_SZ          (21)
-#define JK_LB_NOTES_COUNT            (9)
+#define JK_LB_NOTES_COUNT            (10)
 #define JK_NOTE_LB_FIRST_NAME        ("JK_LB_FIRST_NAME")
 #define JK_NOTE_LB_FIRST_VALUE       ("JK_LB_FIRST_VALUE")
 #define JK_NOTE_LB_FIRST_ACCESSED    ("JK_LB_FIRST_ACCESSED")
+#define JK_NOTE_LB_FIRST_SESSIONS    ("JK_LB_FIRST_SESSIONS")
 #define JK_NOTE_LB_FIRST_READ        ("JK_LB_FIRST_READ")
 #define JK_NOTE_LB_FIRST_TRANSFERRED ("JK_LB_FIRST_TRANSFERRED")
 #define JK_NOTE_LB_FIRST_ERRORS      ("JK_LB_FIRST_ERRORS")
@@ -114,6 +117,7 @@ extern "C"
 #define JK_NOTE_LB_LAST_NAME         ("JK_LB_LAST_NAME")
 #define JK_NOTE_LB_LAST_VALUE        ("JK_LB_LAST_VALUE")
 #define JK_NOTE_LB_LAST_ACCESSED     ("JK_LB_LAST_ACCESSED")
+#define JK_NOTE_LB_LAST_SESSIONS     ("JK_LB_LAST_SESSIONS")
 #define JK_NOTE_LB_LAST_READ         ("JK_LB_LAST_READ")
 #define JK_NOTE_LB_LAST_TRANSFERRED  ("JK_LB_LAST_TRANSFERRED")
 #define JK_NOTE_LB_LAST_ERRORS       ("JK_LB_LAST_ERRORS")
