@@ -19,7 +19,7 @@
  * Description: General purpose map object                                 *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 1042364 $                                          *
+ * Version:     $Revision: 1306287 $                                          *
  ***************************************************************************/
 #if defined(AS400) && !defined(AS400_UTF8)
 #include "apr_xlate.h"
@@ -64,19 +64,7 @@
     }                                          \
 }
 
-struct jk_map
-{
-    jk_pool_t p;
-    jk_pool_atom_t buf[SMALL_POOL_SIZE];
-
-    const char **names;
-    const void **values;
-    unsigned int *keys;
-
-    unsigned int capacity;
-    unsigned int size;
-};
-
+static volatile int global_map_id = 0;
 static void trim_prp_comment(char *prp);
 static size_t trim(char *s);
 static int map_realloc(jk_map_t *m);
@@ -85,7 +73,9 @@ static char *jk_map_replace_properties(jk_map_t *m, jk_map_t *env, char *value);
 int jk_map_alloc(jk_map_t **m)
 {
     if (m) {
-        return jk_map_open(*m = (jk_map_t *)malloc(sizeof(jk_map_t)));
+        *m = (jk_map_t *)calloc(1, sizeof(jk_map_t));
+        if (*m)
+            return jk_map_open(*m);
     }
 
     return JK_FALSE;
@@ -110,6 +100,7 @@ int jk_map_open(jk_map_t *m)
 
     if (m) {
         jk_open_pool(&m->p, m->buf, sizeof(jk_pool_atom_t) * SMALL_POOL_SIZE);
+        m->id = ++global_map_id;
         m->capacity = 0;
         m->size = 0;
         m->keys  = NULL;
@@ -587,7 +578,7 @@ void jk_map_dump(jk_map_t *m, jk_logger_t *l)
             }
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                       "Dump of map: '%s' -> '%s'",
+                       "Dump of map %d: '%s' -> '%s'", m->id,
                        jk_map_name_at(m, i) ? jk_map_name_at(m, i) : "(null)",
                        jk_map_value_at(m, i) ? jk_map_value_at(m, i) : "(null)");
             }
@@ -636,8 +627,8 @@ static size_t trim(char *s)
     if (!(len = strlen(s)))
         return 0;
     for (len = len - 1; (len > 0) &&
-        isspace((int)((unsigned char)s[len])); len--);
-    if ((len > 0) || !isspace((int)((unsigned char)s[len]))) {
+        jk_isspace(s[len]); len--);
+    if ((len > 0) || !jk_isspace(s[len])) {
         len++;
     }
 
@@ -645,7 +636,7 @@ static size_t trim(char *s)
     len++;
 
     for (first = 0; (s[first] != '\0') &&
-        isspace((int)((unsigned char)s[first])); first++);
+        jk_isspace(s[first]); first++);
 
     if (first > 0) {
         memmove(s, s + first, len - first);
@@ -657,28 +648,22 @@ static size_t trim(char *s)
 static int map_realloc(jk_map_t *m)
 {
     if (m->size == m->capacity) {
-        char **names;
-        void **values;
+        const char **names;
+        const void **values;
         unsigned int *keys;
-        int capacity = m->capacity + CAPACITY_INC_SIZE;
+        int capacity  = m->capacity + CAPACITY_INC_SIZE;
+        size_t old_sz = m->capacity * sizeof(void *);
+        size_t new_sz = capacity * sizeof(void *);
 
-        names = (char **)jk_pool_alloc(&m->p, sizeof(char *) * capacity);
-        values = (void **)jk_pool_alloc(&m->p, sizeof(void *) * capacity);
-        keys = (unsigned int *)jk_pool_alloc(&m->p, sizeof(unsigned int) * capacity);
+        names  = (const char  **)jk_pool_realloc(&m->p, new_sz, m->names, old_sz);
+        values = (const void  **)jk_pool_realloc(&m->p, new_sz, m->values, old_sz);
+        keys   = (unsigned int *)jk_pool_realloc(&m->p, new_sz, m->keys, old_sz);
 
-        if (values && names) {
-            if (m->capacity && m->names)
-                memcpy(names, m->names, sizeof(char *) * m->capacity);
+        if (values && names && keys) {
 
-            if (m->capacity && m->values)
-                memcpy(values, m->values, sizeof(void *) * m->capacity);
-
-            if (m->capacity && m->keys)
-                memcpy(keys, m->keys, sizeof(unsigned int) * m->capacity);
-
-            m->names = (const char **)names;
-            m->values = (const void **)values;
-            m->keys = keys;
+            m->names  = names;
+            m->values = values;
+            m->keys   = keys;
             m->capacity = capacity;
 
             return JK_TRUE;
