@@ -22,7 +22,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 1078846 $                                          *
+ * Version:     $Revision: 1188097 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -82,6 +82,11 @@ struct svc_extension
 {
     /* reply_timeout overwrite */
     int reply_timeout;
+    /* Whether to ignore session routing info */
+    int sticky_ignore;
+    /* Whether the request is stateless and should not
+     * influence session load balancing */
+    int stateless;
     /* activation state overwrites for load balancers */
     /* Dynamically allocated array with one entry per lb member. */
     int *activation;
@@ -92,6 +97,10 @@ struct svc_extension
     int *fail_on_status;
     /* Use server error pages for responses >= 400. */
     int use_server_error_pages;
+    /* session_cookie overwrite */
+    char *session_cookie;
+    /* session_path overwrite */
+    char *session_path;
 };
 typedef struct svc_extension svc_extension_t;
 
