@@ -19,7 +19,7 @@
  * Description: Shared Memory support                                      *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 704548 $                                        *
+ * Version:     $Revision: 1334392 $                                        *
  ***************************************************************************/
 
 #include "jk_global.h"
@@ -31,44 +31,48 @@
 #include "jk_ajp14_worker.h"
 #include "jk_shm.h"
 
-/** jk shm header core data structure */
+/** jk shm header core data structure
+ * This is always the first slot in shared memory.
+ */
 struct jk_shm_header_data
 {
     /* Shared memory magic JK_SHM_MAGIC */
-    char   magic[JK_SHM_MAGIC_SIZ];
-    size_t size;
-    size_t pos;
+    char         magic[JK_SHM_MAGIC_SIZ];
+    unsigned int size;
+    unsigned int pos;
     unsigned int childs;
     unsigned int workers;
-    time_t modified;
+    time_t       modified;
 };
 
 typedef struct jk_shm_header_data jk_shm_header_data_t;
 
-/** jk shm header record structure */
+/** jk shm header record structure
+ */
 struct jk_shm_header
 {
     union {
         jk_shm_header_data_t data;
-        char alignbuf[JK_SHM_ALIGN(sizeof(jk_shm_header_data_t))];
+        char alignbuf[JK_SHM_SLOT_SIZE];
     } h;
     char   buf[1];
 };
 
 typedef struct jk_shm_header jk_shm_header_t;
 
-/** jk shm structure */
+/** jk shm in memory structure.
+ */
 struct jk_shm
 {
-    size_t     size;
-    unsigned   ajp_workers;
-    unsigned   lb_sub_workers;
-    unsigned   lb_workers;
-    char       *filename;
-    char       *lockname;
-    int        fd;
-    int        fd_lock;
-    int        attached;
+    unsigned int size;
+    unsigned int ajp_workers;
+    unsigned int lb_sub_workers;
+    unsigned int lb_workers;
+    char        *filename;
+    char        *lockname;
+    int          fd;
+    int          fd_lock;
+    int          attached;
     jk_shm_header_t  *hdr;
     JK_CRIT_SEC       cs;
 };
@@ -83,13 +87,14 @@ static time_t jk_workers_access_time = 0;
 static HANDLE jk_shm_map = NULL;
 static HANDLE jk_shm_hlock = NULL;
 #endif
+static int jk_shm_inited_cs = 0;
 
 /* Calculate needed shm size */
-size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
+int jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
 {
     char **worker_list;
-    unsigned i;
-    unsigned num_of_workers;
+    unsigned int i;
+    unsigned int num_of_workers;
     int num_of_ajp_workers = 0;
     int num_of_lb_sub_workers = 0;
     int num_of_lb_workers = 0;
@@ -103,7 +108,6 @@ size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
         JK_TRACE_EXIT(l);
         return 0;
     }
-
     for (i = 0; i < num_of_workers; i++) {
         const char *type = jk_get_worker_type(init_data, worker_list[i]);
 
@@ -113,7 +117,7 @@ size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
         }
         else if (!strcmp(type, JK_LB_WORKER_NAME)) {
             char **member_list;
-            unsigned num_of_members;
+            unsigned int num_of_members;
             num_of_lb_workers++;
             if (jk_get_lb_worker_list(init_data, worker_list[i],
                                       &member_list, &num_of_members) == JK_FALSE) {
@@ -129,81 +133,131 @@ size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
         }
     }
     if (JK_IS_DEBUG_LEVEL(l))
-        jk_log(l, JK_LOG_DEBUG, "shared memory will contain %d ajp workers of size %d and %d lb workers of size %d with %d members of size %d+%d",
-               num_of_ajp_workers, JK_SHM_AJP_SIZE(1),
-               num_of_lb_workers, JK_SHM_LB_SIZE(1),
-               num_of_lb_sub_workers, JK_SHM_LB_SUB_SIZE(1), JK_SHM_AJP_SIZE(1));
+        jk_log(l, JK_LOG_DEBUG, "shared memory will contain %d ajp workers and %d lb workers with %d members",
+               num_of_ajp_workers,
+               num_of_lb_workers,
+               num_of_lb_sub_workers);
     jk_shmem.ajp_workers = num_of_ajp_workers;
     jk_shmem.lb_sub_workers = num_of_lb_sub_workers;
     jk_shmem.lb_workers = num_of_lb_workers;
     JK_TRACE_EXIT(l);
-    return JK_SHM_AJP_SIZE(jk_shmem.ajp_workers) +
-           JK_SHM_LB_SUB_SIZE(jk_shmem.lb_sub_workers) +
-           JK_SHM_AJP_SIZE(jk_shmem.lb_sub_workers) +
-           JK_SHM_LB_SIZE(jk_shmem.lb_workers);
+    return (JK_SHM_SLOT_SIZE * (jk_shmem.ajp_workers +
+            jk_shmem.lb_sub_workers * 2 +
+            jk_shmem.lb_workers));
 }
 
 
 #if defined (WIN32) || defined(NETWARE)
 
 /* Use plain memory */
-int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
+int jk_shm_open(const char *fname, int sz, jk_logger_t *l)
 {
-    int rc;
+    int rc = -1;
     int attached = 0;
     char lkname[MAX_PATH];
+    char shname[MAX_PATH] = "";
+
     JK_TRACE_ENTER(l);
+    if (!jk_shm_inited_cs) {
+        jk_shm_inited_cs = 1;
+        JK_INIT_CS(&jk_shmem.cs, rc);
+    }
+    JK_ENTER_CS(&jk_shmem.cs);
     if (jk_shmem.hdr) {
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG, "Shared memory is already opened");
         JK_TRACE_EXIT(l);
+        JK_LEAVE_CS(&jk_shmem.cs);
         return 0;
     }
-
-    jk_shmem.size = JK_SHM_ALIGN(sizeof(jk_shm_header_t) + sz);
-
+    if (sz < 0) {
+        jk_log(l, JK_LOG_ERROR, "Invalid shared memory size (%d)", sz);
+        JK_TRACE_EXIT(l);
+        return EINVAL;
+    }
+    jk_shmem.size = JK_SHM_ALIGN(JK_SHM_SLOT_SIZE + sz);
 #if defined (WIN32)
+    jk_shm_map   = NULL;
+    jk_shm_hlock = NULL;
     if (fname) {
-        jk_shm_map = CreateFileMapping(INVALID_HANDLE_VALUE,
-                                       NULL,
-                                       PAGE_READWRITE,
-                                       0,
-                                       (DWORD)(sizeof(jk_shm_header_t) + sz),
-                                       fname);
-        if (GetLastError() == ERROR_ALREADY_EXISTS) {
-            attached = 1;
-            if (jk_shm_map == NULL || jk_shm_map == INVALID_HANDLE_VALUE) {
-                jk_shm_map = OpenFileMapping(PAGE_READWRITE, FALSE, fname);
+        int i;
+        SIZE_T shmsz = 0;
+        strcpy(shname, "Global\\");
+        strncat(shname, fname, MAX_PATH - 8);
+        for(i = 7; i < (int)strlen(shname); i++) {
+            if (!jk_isalnum(shname[i]))
+                shname[i] = '_';
+            else
+                shname[i] = toupper(shname[i]);
+        }
+        strcpy(lkname, shname);
+        strncat(lkname, "_MUTEX", MAX_PATH - 1);
+        jk_shm_hlock = CreateMutex(jk_get_sa_with_null_dacl(), TRUE, lkname);
+        if (jk_shm_hlock == NULL) {
+            if (GetLastError() == ERROR_ALREADY_EXISTS) {
+                attached = 1;
+                jk_shm_hlock = OpenMutex(MUTEX_ALL_ACCESS, FALSE, lkname);
             }
         }
-        if (jk_shm_map == NULL || jk_shm_map == INVALID_HANDLE_VALUE) {
+        else if (GetLastError() == ERROR_ALREADY_EXISTS)
+            attached = 1;
+        if (jk_shm_hlock == NULL) {
+            rc = GetLastError();
+            jk_log(l, JK_LOG_ERROR, "Failed to open shared memory mutex %s with errno=%d",
+                   lkname, rc);
+            JK_LEAVE_CS(&jk_shmem.cs);
             JK_TRACE_EXIT(l);
-            return -1;
+            return rc;
         }
-        sprintf(lkname, "Global\\%s_MUTEX", fname);
         if (attached) {
-            jk_shm_hlock = OpenMutex(MUTEX_ALL_ACCESS, FALSE, lkname);
+            DWORD ws = WaitForSingleObject(jk_shm_hlock, INFINITE);
+            if (ws == WAIT_FAILED) {
+                rc = GetLastError();
+                CloseHandle(jk_shm_hlock);
+                jk_shm_hlock = NULL;
+                JK_LEAVE_CS(&jk_shmem.cs);
+                JK_TRACE_EXIT(l);
+                return rc;
+            }
+            jk_shm_map = OpenFileMapping(FILE_MAP_READ | FILE_MAP_WRITE, FALSE, shname);
+            if (jk_shm_map == NULL) {
+                rc = GetLastError();
+                jk_log(l, JK_LOG_ERROR, "Failed to open shared memory %s with errno=%d",
+                       shname, rc);
+            }
         }
-        else {
-            jk_shm_hlock = CreateMutex(NULL, FALSE, lkname);            
+        if (jk_shm_map == NULL) {
+            shmsz = jk_shmem.size;
+            jk_shm_map = CreateFileMapping(INVALID_HANDLE_VALUE,
+                                           jk_get_sa_with_null_dacl(),
+                                           PAGE_READWRITE,
+                                           0,
+                                           (DWORD)shmsz,
+                                           shname);
         }
-        if (jk_shm_hlock == NULL || jk_shm_hlock == INVALID_HANDLE_VALUE) {
-            CloseHandle(jk_shm_map);
-            jk_shm_map = NULL;
+        if (jk_shm_map == NULL || jk_shm_map == INVALID_HANDLE_VALUE) {
+            rc = GetLastError();
+            jk_log(l, JK_LOG_ERROR, "Failed to map shared memory %s with errno=%d",
+                   fname, rc);
+            CloseHandle(jk_shm_hlock);
+            jk_shm_hlock = NULL;
+            jk_shm_map   = NULL;
+            JK_LEAVE_CS(&jk_shmem.cs);
             JK_TRACE_EXIT(l);
-            return -1;
+            return rc;
         }
         jk_shmem.hdr = (jk_shm_header_t *)MapViewOfFile(jk_shm_map,
-                                                        FILE_MAP_ALL_ACCESS,
+                                                        FILE_MAP_READ | FILE_MAP_WRITE,
                                                         0,
                                                         0,
-                                                        0);
+                                                        shmsz);
     }
     else
 #endif
     jk_shmem.hdr = (jk_shm_header_t *)calloc(1, jk_shmem.size);
     if (!jk_shmem.hdr) {
 #if defined (WIN32)
+        rc = GetLastError();
         if (jk_shm_map) {
             CloseHandle(jk_shm_map);
             jk_shm_map = NULL;
@@ -213,18 +267,20 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
             jk_shm_hlock = NULL;
         }
 #endif
+        JK_LEAVE_CS(&jk_shmem.cs);
         JK_TRACE_EXIT(l);
-        return -1;
+        return rc;
     }
     if (!jk_shmem.filename) {
-        if (fname)
-            jk_shmem.filename = strdup(fname);
+        if (shname[0])
+            jk_shmem.filename = strdup(shname);
         else
             jk_shmem.filename = strdup("memory");
     }
     jk_shmem.fd       = 0;
     jk_shmem.attached = attached;
     if (!attached) {
+        memset(jk_shmem.hdr, 0, jk_shmem.size);
         memcpy(jk_shmem.hdr->h.data.magic, shm_signature,
                JK_SHM_MAGIC_SIZ);
         jk_shmem.hdr->h.data.size = sz;
@@ -239,6 +295,7 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
          * if the number of workers change between
          * open and attach or between two attach operations.
          */
+#if 0
         if (jk_shmem.hdr->h.data.childs > 1) {
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
@@ -248,20 +305,29 @@ int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
         }
         jk_shmem.hdr->h.data.pos     = 0;
         jk_shmem.hdr->h.data.workers = 0;
+#endif
     }
-    JK_INIT_CS(&(jk_shmem.cs), rc);
+#if defined (WIN32)
+    if (jk_shm_hlock != NULL) {
+        /* Unlock shared memory */
+        ReleaseMutex(jk_shm_hlock);
+    }
+#endif
+    JK_LEAVE_CS(&jk_shmem.cs);
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
-               "%s shared memory %s size=%u free=%u addr=%#lx",
+               "%s shared memory %s [%d] size=%u workers=%d free=%u addr=%#lx",
                attached ? "Attached" : "Initialized",
-               jk_shm_name(), jk_shmem.size,
+               jk_shm_name(),
+               jk_shmem.hdr->h.data.childs,
+               jk_shmem.size, jk_shmem.hdr->h.data.workers - 1,
                jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
                jk_shmem.hdr);
     JK_TRACE_EXIT(l);
     return 0;
 }
 
-int jk_shm_attach(const char *fname, size_t sz, jk_logger_t *l)
+int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
 {
     JK_TRACE_ENTER(l);
     if (!jk_shm_open(fname, sz, l)) {
@@ -284,12 +350,21 @@ int jk_shm_attach(const char *fname, size_t sz, jk_logger_t *l)
     }
 }
 
-void jk_shm_close()
+void jk_shm_close(jk_logger_t *l)
 {
+    if (jk_shm_inited_cs) {
+        JK_ENTER_CS(&jk_shmem.cs);
+    }
     if (jk_shmem.hdr) {
-        int rc;
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "Closed shared memory %s childs=%u",
+                   jk_shm_name(), jk_shmem.hdr->h.data.childs);
+        }
 #if defined (WIN32)
         if (jk_shm_hlock) {
+            WaitForSingleObject(jk_shm_hlock, 60000);
+            ReleaseMutex(jk_shm_hlock);
             CloseHandle(jk_shm_hlock);
             jk_shm_hlock = NULL;
         }
@@ -302,13 +377,15 @@ void jk_shm_close()
         else
 #endif
         free(jk_shmem.hdr);
-        JK_DELETE_CS(&(jk_shmem.cs), rc);
     }
     jk_shmem.hdr = NULL;
     if (jk_shmem.filename) {
         free(jk_shmem.filename);
         jk_shmem.filename = NULL;
     }
+    if (jk_shm_inited_cs) {
+        JK_LEAVE_CS(&jk_shmem.cs);
+    }
 }
 
 #else
@@ -416,7 +493,7 @@ static int do_shm_open_lock(const char *fname, int attached, jk_logger_t *l)
 }
 
 static int do_shm_open(const char *fname, int attached,
-                       size_t sz, jk_logger_t *l)
+                       int sz, jk_logger_t *l)
 {
     int rc;
     int fd;
@@ -426,6 +503,10 @@ static int do_shm_open(const char *fname, int attached,
 #endif
 
     JK_TRACE_ENTER(l);
+    if (!jk_shm_inited_cs) {
+        jk_shm_inited_cs = 1;
+        JK_INIT_CS(&jk_shmem.cs, rc);
+    }
     if (jk_shmem.hdr) {
         /* Probably a call from vhost */
         if (!attached)
@@ -438,7 +519,12 @@ static int do_shm_open(const char *fname, int attached,
         JK_TRACE_EXIT(l);
         return 0;
     }
-    jk_shmem.size = JK_SHM_ALIGN(sizeof(jk_shm_header_t) + sz);
+    if (sz < 0) {
+        jk_log(l, JK_LOG_ERROR, "Invalid shared memory size (%d)", sz);
+        JK_TRACE_EXIT(l);
+        return EINVAL;
+    }
+    jk_shmem.size = JK_SHM_ALIGN(JK_SHM_SLOT_SIZE + sz);
 
     if (!fname) {
         /* Use plain memory in case there is no file name */
@@ -542,16 +628,17 @@ static int do_shm_open(const char *fname, int attached,
                    jk_shmem.hdr);
     }
     else {
-        unsigned int nchild;
         jk_shmem.hdr->h.data.childs++;
         jk_shmem.attached = (int)getpid();
-        nchild = jk_shmem.hdr->h.data.childs;
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
-                   "Attached shared memory %s [%d] size=%u free=%u addr=%#lx",
-                   jk_shm_name(), nchild, jk_shmem.size,
+                   "Attached shared memory %s [%d] size=%u workers=%u free=%u addr=%#lx",
+                   jk_shm_name(),
+                   jk_shmem.hdr->h.data.childs,
+                   jk_shmem.size, jk_shmem.hdr->h.data.workers - 1,
                    jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
                    jk_shmem.hdr);
+#if 0
         /*
          * Reset the shared memory so that
          * alloc works even for attached memory.
@@ -568,8 +655,8 @@ static int do_shm_open(const char *fname, int attached,
         }
         jk_shmem.hdr->h.data.pos     = 0;
         jk_shmem.hdr->h.data.workers = 0;
+#endif
     }
-    JK_INIT_CS(&(jk_shmem.cs), rc);
     if ((rc = do_shm_open_lock(jk_shmem.filename, attached, l))) {
         if (!attached) {
             munmap((void *)jk_shmem.hdr, jk_shmem.size);
@@ -592,24 +679,28 @@ static int do_shm_open(const char *fname, int attached,
     return 0;
 }
 
-int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l)
+int jk_shm_open(const char *fname, int sz, jk_logger_t *l)
 {
     return do_shm_open(fname, 0, sz, l);
 }
 
-int jk_shm_attach(const char *fname, size_t sz, jk_logger_t *l)
+int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
 {
     return do_shm_open(fname, 1, sz, l);
 }
 
-void jk_shm_close()
+void jk_shm_close(jk_logger_t *l)
 {
-    int rc;
 #ifdef AS400_UTF8
     char *wptr;
 #endif
 
     if (jk_shmem.hdr) {
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "Closed shared memory %s childs=%u",
+                   jk_shm_name(), jk_shmem.hdr->h.data.childs);
+        }
         --jk_shmem.hdr->h.data.childs;
 
 #ifdef JK_SHM_LOCK_REOPEN
@@ -618,7 +709,6 @@ void jk_shm_close()
             jk_shmem.fd_lock = -1;
         }
 #endif
-        JK_DELETE_CS(&(jk_shmem.cs), rc);
         if (jk_shmem.attached) {
             int p = (int)getpid();
             if (p == jk_shmem.attached) {
@@ -672,23 +762,73 @@ void jk_shm_close()
 
 #endif
 
-void *jk_shm_alloc(jk_pool_t *p, size_t size)
+void *jk_shm_alloc(jk_pool_t *p)
 {
     void *rc = NULL;
 
     if (jk_shmem.hdr) {
-        size = JK_SHM_ALIGN(size);
-        if ((jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos) >= size) {
+        jk_shm_lock();
+        if ((jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos) >= JK_SHM_SLOT_SIZE) {
             rc = &(jk_shmem.hdr->buf[jk_shmem.hdr->h.data.pos]);
-            jk_shmem.hdr->h.data.pos += size;
+            jk_shmem.hdr->h.data.pos += JK_SHM_SLOT_SIZE;
         }
+        jk_shm_unlock();
     }
     else if (p)
-        rc = jk_pool_alloc(p, size);
+        rc = jk_pool_alloc(p, JK_SHM_SLOT_SIZE);
 
     return rc;
 }
 
+jk_shm_worker_header_t *jk_shm_alloc_worker(jk_pool_t *p, int type,
+                                            int parent_id, const char *name)
+{
+    unsigned int i;
+    jk_shm_worker_header_t *w = 0;
+
+    if (jk_shmem.hdr) {
+        jk_shm_lock();
+        for (i = 0; i < jk_shmem.hdr->h.data.pos; i += JK_SHM_SLOT_SIZE) {
+            w = (jk_shm_worker_header_t *)(jk_shmem.hdr->buf + i);
+            if (w->type == type && w-> parent_id == parent_id &&
+                strcmp(w->name, name) == 0) {
+                /* We have found already created worker */
+                jk_shm_unlock();
+                return w;
+            }
+        }
+        /* Allocate new worker */
+        if ((jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos) >= JK_SHM_SLOT_SIZE) {
+            w = (jk_shm_worker_header_t *)(jk_shmem.hdr->buf + jk_shmem.hdr->h.data.pos);
+            memset(w, 0, JK_SHM_SLOT_SIZE);
+            strncpy(w->name, name, JK_SHM_STR_SIZ);
+            jk_shmem.hdr->h.data.workers++;
+            w->id = jk_shmem.hdr->h.data.workers;
+            w->type = type;
+            w->parent_id = parent_id;
+            jk_shmem.hdr->h.data.pos += JK_SHM_SLOT_SIZE;
+        }
+        else {
+            /* No more free memory left.
+             */
+            w = NULL;
+        }
+        jk_shm_unlock();
+    }
+    else if (p) {
+        w = (jk_shm_worker_header_t *)jk_pool_alloc(p, JK_SHM_SLOT_SIZE);
+        if (w) {
+            memset(w, 0, JK_SHM_SLOT_SIZE);
+            strncpy(w->name, name, JK_SHM_STR_SIZ);
+            w->id = 0;
+            w->type = type;
+            w->parent_id = parent_id;
+        }
+    }
+
+    return w;
+}
+
 const char *jk_shm_name()
 {
     return jk_shmem.filename;
@@ -705,11 +845,13 @@ time_t jk_shm_get_workers_time()
 
 void jk_shm_set_workers_time(time_t t)
 {
+    jk_shm_lock();
     if (jk_shmem.hdr)
         jk_shmem.hdr->h.data.modified = t;
     else
         jk_workers_modified_time = t;
     jk_workers_access_time = t;
+    jk_shm_unlock();
 }
 
 int jk_shm_is_modified()
@@ -728,18 +870,18 @@ void jk_shm_sync_access_time()
 
 int jk_shm_lock()
 {
-    int rc;
-    JK_ENTER_CS(&(jk_shmem.cs), rc);
+    int rc = JK_TRUE;
+
+    if (!jk_shm_inited_cs)
+        return JK_FALSE;
+    JK_ENTER_CS(&jk_shmem.cs);
 #if defined (WIN32)
-    if (rc == JK_TRUE && jk_shm_hlock != NULL) {
-        DWORD rv = WaitForSingleObject(jk_shm_hlock, INFINITE);
-        if (rv == WAIT_OBJECT_0 || rv == WAIT_ABANDONED)
-            rc = JK_TRUE;
-        else
-            rc = JK_FALSE;         
+    if (jk_shm_hlock != NULL) {
+        if (WaitForSingleObject(jk_shm_hlock, INFINITE) == WAIT_FAILED)
+            rc = JK_FALSE;
     }
-#else    
-    if (rc == JK_TRUE && jk_shmem.fd_lock != -1) {
+#else
+    if (jk_shmem.fd_lock != -1) {
         JK_ENTER_LOCK(jk_shmem.fd_lock, rc);
     }
 #endif
@@ -748,65 +890,37 @@ int jk_shm_lock()
 
 int jk_shm_unlock()
 {
-    int rc;
-    JK_LEAVE_CS(&(jk_shmem.cs), rc);
+    int rc = JK_TRUE;
+
+    if (!jk_shm_inited_cs)
+        return JK_FALSE;
 #if defined (WIN32)
-    if (rc == JK_TRUE && jk_shm_hlock != NULL) {
-        if (!ReleaseMutex(jk_shm_hlock))
-            rc = JK_FALSE;         
+    if (jk_shm_hlock != NULL) {
+        ReleaseMutex(jk_shm_hlock);
     }
-#else    
-    if (rc == JK_TRUE && jk_shmem.fd_lock != -1) {
+#else
+    if (jk_shmem.fd_lock != -1) {
         JK_LEAVE_LOCK(jk_shmem.fd_lock, rc);
     }
 #endif
+    JK_LEAVE_CS(&jk_shmem.cs);
     return rc;
 }
 
-jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p)
+jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p, const char *name)
 {
-    jk_shm_ajp_worker_t *w = (jk_shm_ajp_worker_t *)jk_shm_alloc(p, JK_SHM_AJP_WORKER_SIZE);
-    if (w) {
-        memset(w, 0, JK_SHM_AJP_WORKER_SIZE);
-        if (jk_shmem.hdr) {
-            jk_shmem.hdr->h.data.workers++;
-            w->h.id = jk_shmem.hdr->h.data.workers;
-            w->h.type = JK_AJP13_WORKER_TYPE;
-        }
-        else
-            w->h.id = -1;
-    }
-    return w;
+    return (jk_shm_ajp_worker_t *)jk_shm_alloc_worker(p,
+                                    JK_AJP13_WORKER_TYPE, 0, name);
 }
 
-jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p)
+jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p, int lb_id, const char *name)
 {
-    jk_shm_lb_sub_worker_t *w = (jk_shm_lb_sub_worker_t *)jk_shm_alloc(p, JK_SHM_LB_SUB_WORKER_SIZE);
-    if (w) {
-        memset(w, 0, JK_SHM_LB_SUB_WORKER_SIZE);
-        if (jk_shmem.hdr) {
-            jk_shmem.hdr->h.data.workers++;
-            w->h.id = jk_shmem.hdr->h.data.workers;
-            w->h.type = JK_LB_SUB_WORKER_TYPE;
-        }
-        else
-            w->h.id = -1;
-    }
-    return w;
+    return (jk_shm_lb_sub_worker_t *)jk_shm_alloc_worker(p,
+                                    JK_LB_SUB_WORKER_TYPE, lb_id, name);
 }
 
-jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p)
+jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p, const char *name)
 {
-    jk_shm_lb_worker_t *w = (jk_shm_lb_worker_t *)jk_shm_alloc(p, JK_SHM_LB_WORKER_SIZE);
-    if (w) {
-        memset(w, 0, JK_SHM_LB_WORKER_SIZE);
-        if (jk_shmem.hdr) {
-            jk_shmem.hdr->h.data.workers++;
-            w->h.id = jk_shmem.hdr->h.data.workers;
-            w->h.type = JK_LB_WORKER_TYPE;
-        }
-        else
-            w->h.id = -1;
-    }
-    return w;
+    return (jk_shm_lb_worker_t *)jk_shm_alloc_worker(p,
+                                    JK_LB_WORKER_TYPE, 0, name);
 }
