@@ -19,7 +19,7 @@
  * Description: Shared Memory object header file                           *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 893253 $                                           *
+ * Version:     $Revision: 1334348 $                                           *
  ***************************************************************************/
 #ifndef _JK_SHM_H
 #define _JK_SHM_H
@@ -48,27 +48,29 @@ extern "C"
 #define JK_SHM_MAGIC    '!', 'J', 'K', 'S', 'H', 'M', JK_SHM_MAJOR, JK_SHM_MINOR
 #define JK_SHM_MAGIC_SIZ  8
 
+/* XXX: Check if adding struct members for overflow */
+#define JK_SHM_SLOT_SIZE          384
 /* Really huge numbers, but 64 workers should be enough */
 #define JK_SHM_MAX_WORKERS        64
-#define JK_SHM_ALIGNMENT          64
+#define JK_SHM_ALIGNMENT          JK_SHM_SLOT_SIZE
 #define JK_SHM_ALIGN(x)           JK_ALIGN((x), JK_SHM_ALIGNMENT)
-#define JK_SHM_AJP_WORKER_SIZE    JK_SHM_ALIGN(sizeof(jk_shm_ajp_worker_t))
-#define JK_SHM_LB_SUB_WORKER_SIZE JK_SHM_ALIGN(sizeof(jk_shm_lb_sub_worker_t))
-#define JK_SHM_LB_WORKER_SIZE     JK_SHM_ALIGN(sizeof(jk_shm_lb_worker_t))
-#define JK_SHM_AJP_SIZE(x)        ((x) * JK_SHM_AJP_WORKER_SIZE)
-#define JK_SHM_LB_SUB_SIZE(x)     ((x) * JK_SHM_LB_SUB_WORKER_SIZE)
-#define JK_SHM_LB_SIZE(x)         ((x) * JK_SHM_LB_WORKER_SIZE)
-#define JK_SHM_DEF_SIZE           JK_SHM_AJP_SIZE(JK_SHM_MAX_WORKERS) + JK_SHM_LB_SUB_SIZE(JK_SHM_MAX_WORKERS) + JK_SHM_LB_SIZE(JK_SHM_MAX_WORKERS)
+#define JK_SHM_DEF_SIZE           ((JK_SHM_SLOT_SIZE * JK_SHM_MAX_WORKERS * 3) + JK_SHM_ALIGNMENT)
 
 /** jk shm generic worker record structure */
 struct jk_shm_worker_header
 {
+    /* Shared memory slot id */
     int     id;
+    /* JK_XXX_WORKER_TYPE */
     int     type;
     /* worker name */
     char    name[JK_SHM_STR_SIZ+1];
+    /* parent slot id.
+     * Zero in case worker does not belong to balancer.
+     */
+    int     parent_id;
     /* Sequence counter starting at 0 and increasing
-     * every time we change the config
+     * every time we change the config.
      */
     volatile unsigned int sequence;
 };
@@ -151,6 +153,8 @@ struct jk_shm_lb_sub_worker
     volatile time_t error_time;
     /* Number of times the worker was elected - snapshot during maintenance */
     volatile jk_uint64_t  elected_snapshot;
+    /* Number of non-sticky requests handled, that were not marked as stateless */
+    volatile jk_uint64_t  sessions;
     /* Number of non 200 responses */
     volatile jk_uint32_t  errors;
 };
@@ -189,40 +193,47 @@ typedef struct jk_shm_lb_worker jk_shm_lb_worker_t;
 const char *jk_shm_name(void);
 
 /* Calculate needed shm size */
-size_t jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l);
+int jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l);
 
 /* Open the shared memory creating file if needed
  */
-int jk_shm_open(const char *fname, size_t sz, jk_logger_t *l);
+int jk_shm_open(const char *fname, int sz, jk_logger_t *l);
 
 /* Close the shared memory
  */
-void jk_shm_close(void);
+void jk_shm_close(jk_logger_t *l);
 
 /* Attach the shared memory in child process.
  * File has to be opened in parent.
  */
-int jk_shm_attach(const char *fname, size_t sz, jk_logger_t *l);
+int jk_shm_attach(const char *fname, int sz, jk_logger_t *l);
 
 /* allocate shm memory
  * If there is no shm present the pool will be used instead
  */
-void *jk_shm_alloc(jk_pool_t *p, size_t size);
+void *jk_shm_alloc(jk_pool_t *p);
+
+/* allocate shm memory
+ * If there is no shm present the pool will be used instead
+ */
+jk_shm_worker_header_t *jk_shm_alloc_worker(jk_pool_t *p, int type,
+                                            int parent_id, const char *name);
 
 /* allocate shm ajp worker record
  * If there is no shm present the pool will be used instead
  */
-jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p);
+jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p, const char *name);
 
 /* allocate shm lb sub worker record
  * If there is no shm present the pool will be used instead
  */
-jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p);
+jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p,
+                                                   int lb_id, const char *name);
 
 /* allocate shm lb worker record
  * If there is no shm present the pool will be used instead
  */
-jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p);
+jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p, const char *name);
 
 /* Return workers.properties last modified time
  */
