@@ -20,7 +20,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 890739 $                                          *
+ * Version:     $Revision: 1334403 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -40,11 +40,15 @@
 #endif
 
 #define JK_UWMAP_EXTENSION_REPLY_TIMEOUT  "reply_timeout="
+#define JK_UWMAP_EXTENSION_STICKY_IGNORE  "sticky_ignore="
+#define JK_UWMAP_EXTENSION_STATELESS      "stateless="
 #define JK_UWMAP_EXTENSION_ACTIVE         "active="
 #define JK_UWMAP_EXTENSION_DISABLED       "disabled="
 #define JK_UWMAP_EXTENSION_STOPPED        "stopped="
 #define JK_UWMAP_EXTENSION_FAIL_ON_STATUS "fail_on_status="
 #define JK_UWMAP_EXTENSION_USE_SRV_ERRORS "use_server_errors="
+#define JK_UWMAP_EXTENSION_SESSION_COOKIE "session_cookie="
+#define JK_UWMAP_EXTENSION_SESSION_PATH   "session_path="
 
 #define IND_SWITCH(x)                      (((x)+1) % 2)
 #define IND_THIS(x)                        ((x)[uw_map->index])
@@ -52,6 +56,8 @@
 
 #define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
 
+static volatile int map_id_counter = 0;
+
 static const char *uri_worker_map_source_type[] = {
     "unknown",
     SOURCE_TYPE_TEXT_WORKERDEF,
@@ -165,30 +171,34 @@ static void uri_worker_map_dump(jk_uri_worker_map_t *uw_map,
 {
     JK_TRACE_ENTER(l);
     if (uw_map) {
-        int i, off, k;
-        uri_worker_record_t *uwr = NULL;
-        char buf[32];
-        jk_log(l, JK_LOG_DEBUG, "uri map dump %s: index=%d file='%s' reject_unsafe=%d "
-               "reload=%d modified=%d checked=%d",
-               reason, uw_map->index, STRNULL_FOR_NULL(uw_map->fname), uw_map->reject_unsafe,
-               uw_map->reload, uw_map->modified, uw_map->checked);
-        for(i=0;i<=1;i++) {
+        int i, off;
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG, "uri map dump %s: id=%d, index=%d file='%s' reject_unsafe=%d "
+                  "reload=%d modified=%d checked=%d",
+                   reason, uw_map->id, uw_map->index, STRNULL_FOR_NULL(uw_map->fname),
+                   uw_map->reject_unsafe, uw_map->reload, uw_map->modified, uw_map->checked);
+        }
+        for (i = 0; i <= 1; i++) {
             jk_log(l, JK_LOG_DEBUG, "generation %d: size=%d nosize=%d capacity=%d",
                    i, uw_map->size[i], uw_map->nosize[i], uw_map->capacity[i], uw_map->maps[i]);
         }
 
         off = uw_map->index;
-        for(i=0;i<=1;i++) {
+        for (i = 0; i <= 1; i++) {
+            char buf[32];
+            int k;
             unsigned int j;
             k = (i + off) % 2;
             for (j = 0; j < uw_map->size[k]; j++) {
-                uwr = uw_map->maps[k][j];
-                jk_log(l, JK_LOG_DEBUG, "%s (%d) map #%d: uri=%s worker=%s context=%s "
-                       "source=%s type=%s len=%d",
-                       i ? "NEXT" : "THIS", i, j,
-                       STRNULL_FOR_NULL(uwr->uri), STRNULL_FOR_NULL(uwr->worker_name),
-                       STRNULL_FOR_NULL(uwr->context), STRNULL_FOR_NULL(uri_worker_map_get_source(uwr,l)),
-                       STRNULL_FOR_NULL(uri_worker_map_get_match(uwr,buf,l)), uwr->context_len);
+                uri_worker_record_t *uwr = uw_map->maps[k][j];
+                if (uwr && JK_IS_DEBUG_LEVEL(l)) {
+                    jk_log(l, JK_LOG_DEBUG, "%s (%d) map #%d: uri=%s worker=%s context=%s "
+                           "source=%s type=%s len=%d",
+                           i ? "NEXT" : "THIS", i, j,
+                           STRNULL_FOR_NULL(uwr->uri), STRNULL_FOR_NULL(uwr->worker_name),
+                           STRNULL_FOR_NULL(uwr->context), STRNULL_FOR_NULL(uri_worker_map_get_source(uwr,l)),
+                           STRNULL_FOR_NULL(uri_worker_map_get_match(uwr,buf,l)), uwr->context_len);
+                }
             }
         }
     }
@@ -220,7 +230,7 @@ int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
 
         jk_open_pool(&(uw_map->p),
                      uw_map->buf, sizeof(jk_pool_atom_t) * BIG_POOL_SIZE);
-        for(i=0;i<=1;i++) {
+        for (i = 0; i <= 1; i++) {
             jk_open_pool(&(uw_map->p_dyn[i]),
                          uw_map->buf_dyn[i], sizeof(jk_pool_atom_t) * BIG_POOL_SIZE);
             uw_map->size[i] = 0;
@@ -228,6 +238,7 @@ int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
             uw_map->capacity[i] = 0;
             uw_map->maps[i] = NULL;
         }
+        uw_map->id = 0;
         uw_map->index = 0;
         uw_map->fname = NULL;
         uw_map->reject_unsafe = 0;
@@ -237,6 +248,8 @@ int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
 
         if (init_data)
             rc = uri_worker_map_open(uw_map, init_data, l);
+        if (rc == JK_TRUE)
+            uw_map->id = ++map_id_counter;
         JK_TRACE_EXIT(l);
         return rc;
     }
@@ -252,8 +265,7 @@ static int uri_worker_map_close(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
     JK_TRACE_ENTER(l);
 
     if (uw_map) {
-        int i;
-        JK_DELETE_CS(&(uw_map->cs), i);
+        JK_DELETE_CS(&uw_map->cs);
         jk_close_pool(&uw_map->p_dyn[0]);
         jk_close_pool(&uw_map->p_dyn[1]);
         jk_close_pool(&uw_map->p);
@@ -359,8 +371,7 @@ static int uri_worker_map_clear(jk_uri_worker_map_t *uw_map,
     return JK_TRUE;
 }
 
-static void extract_activation(jk_uri_worker_map_t *uw_map,
-                               uri_worker_record_t *uwr,
+static void extract_activation(jk_pool_t *p,
                                lb_worker_t *lb,
                                int *activations,
                                char *workers,
@@ -368,7 +379,6 @@ static void extract_activation(jk_uri_worker_map_t *uw_map,
                                jk_logger_t *l)
 {
     unsigned int i;
-    jk_pool_t *p;
     char *worker;
 #ifdef _MT_CODE_PTHREAD
     char *lasts;
@@ -376,10 +386,6 @@ static void extract_activation(jk_uri_worker_map_t *uw_map,
 
     JK_TRACE_ENTER(l);
 
-    if (uwr->source_type == SOURCE_TYPE_URIMAP)
-        p = &IND_NEXT(uw_map->p_dyn);
-    else
-        p = &uw_map->p;
     worker = jk_pool_strdup(p, workers);
 
 #ifdef _MT_CODE_PTHREAD
@@ -388,7 +394,7 @@ static void extract_activation(jk_uri_worker_map_t *uw_map,
 #else
     for (worker = strtok(worker, ", "); worker; worker = strtok(NULL, ", ")) {
 #endif
-        for (i=0; i<lb->num_of_workers; i++) {
+        for (i = 0; i < lb->num_of_workers; i++) {
             if (!strcmp(worker, lb->lb_workers[i].name)) {
                 if (activations[i] != JK_LB_ACTIVATION_UNSET)
                     jk_log(l, JK_LOG_WARNING,
@@ -411,14 +417,14 @@ static void extract_activation(jk_uri_worker_map_t *uw_map,
 
 }
 
-static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
-                                   uri_worker_record_t *uwr,
-                                   jk_logger_t *l)
+static void extension_fix_fail_on_status(jk_pool_t *p,
+                                         const char *name,
+                                         rule_extension_t *extensions,
+                                         jk_logger_t *l)
 {
     unsigned int i;
     int j;
     int cnt = 1;
-    jk_pool_t *p;
     char *status;
 #ifdef _MT_CODE_PTHREAD
     char *lasts;
@@ -426,33 +432,30 @@ static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
 
     JK_TRACE_ENTER(l);
 
-    for (i=0; i<strlen(uwr->extensions.fail_on_status_str); i++) {
-        if (uwr->extensions.fail_on_status_str[i] == ',' ||
-            uwr->extensions.fail_on_status_str[i] == ' ')
+    for (i = 0; i < (unsigned int)strlen(extensions->fail_on_status_str); i++) {
+        if (extensions->fail_on_status_str[i] == ',' ||
+            extensions->fail_on_status_str[i] == ' ')
             cnt++;
     }
-    uwr->extensions.fail_on_status_size = cnt;
+    extensions->fail_on_status_size = cnt;
 
-    if (uwr->source_type == SOURCE_TYPE_URIMAP)
-        p = &IND_NEXT(uw_map->p_dyn);
-    else
-        p = &uw_map->p;
-    status = jk_pool_strdup(p, uwr->extensions.fail_on_status_str);
-    uwr->extensions.fail_on_status = (int *)jk_pool_alloc(p,
-                                            uwr->extensions.fail_on_status_size * sizeof(int));
-    if (!uwr->extensions.fail_on_status) {
+    status = jk_pool_strdup(p, extensions->fail_on_status_str);
+    extensions->fail_on_status = (int *)jk_pool_alloc(p,
+                                            extensions->fail_on_status_size * sizeof(int));
+    if (!extensions->fail_on_status) {
         jk_log(l, JK_LOG_ERROR,
-               "can't alloc extensions fail_on_status list");
+               "can't alloc extensions fail_on_status list for worker (%s)",
+               name);
         JK_TRACE_EXIT(l);
         return;
     } else if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
-               "Allocated fail_on_status array of size %d for worker %s",
-               uwr->extensions.fail_on_status_size, uwr->worker_name);
+               "Allocated fail_on_status array of size %d for worker (%s)",
+               extensions->fail_on_status_size, name);
 
 
-    for (j=0; j<uwr->extensions.fail_on_status_size; j++) {
-        uwr->extensions.fail_on_status[j] = 0;
+    for (j=0; j<extensions->fail_on_status_size; j++) {
+        extensions->fail_on_status[j] = 0;
     }
 
     cnt = 0;
@@ -462,7 +465,7 @@ static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
 #else
     for (status = strtok(status, ", "); status; status = strtok(NULL, ", ")) {
 #endif
-        uwr->extensions.fail_on_status[cnt] = atoi(status);
+        extensions->fail_on_status[cnt] = atoi(status);
         cnt++;
     }
 
@@ -470,6 +473,107 @@ static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
 
 }
 
+static int extension_fix_activation(jk_pool_t *p, const char *name, jk_worker_t *jw,
+                                    rule_extension_t *extensions, jk_logger_t *l)
+{
+
+    JK_TRACE_ENTER(l);
+
+    if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG,
+               "Checking extension for worker %s of type %s (%d)",
+               name, wc_get_name_for_type(jw->type,l), jw->type);
+
+    if (jw->type == JK_LB_WORKER_TYPE &&
+        (extensions->active || extensions->disabled || extensions->stopped)) {
+        int j;
+        lb_worker_t *lb = (lb_worker_t *)jw->worker_private;
+        if (!extensions->activation) {
+            extensions->activation_size = lb->num_of_workers;
+            extensions->activation = (int *)jk_pool_alloc(p,
+                                                    extensions->activation_size * sizeof(int));
+            if (!extensions->activation) {
+                jk_log(l, JK_LOG_ERROR,
+                       "can't alloc extensions activation list");
+                JK_TRACE_EXIT(l);
+                return JK_FALSE;
+            }
+            else if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "Allocated activations array of size %d for lb worker %s",
+                       extensions->activation_size, name);
+            for (j=0; j<extensions->activation_size; j++) {
+                extensions->activation[j] = JK_LB_ACTIVATION_UNSET;
+            }
+        }
+        if (extensions->active)
+            extract_activation(p, lb, extensions->activation,
+                               extensions->active, JK_LB_ACTIVATION_ACTIVE, l);
+        if (extensions->disabled)
+            extract_activation(p, lb, extensions->activation,
+                               extensions->disabled, JK_LB_ACTIVATION_DISABLED, l);
+        if (extensions->stopped)
+            extract_activation(p, lb, extensions->activation,
+                               extensions->stopped, JK_LB_ACTIVATION_STOPPED, l);
+    }
+    else if (extensions->active) {
+        jk_log(l, JK_LOG_WARNING,
+               "Worker %s is not of type lb, activation extension "
+               JK_UWMAP_EXTENSION_ACTIVE " for %s ignored",
+               name, extensions->active);
+    }
+    else if (extensions->disabled) {
+        jk_log(l, JK_LOG_WARNING,
+               "Worker %s is not of type lb, activation extension "
+               JK_UWMAP_EXTENSION_DISABLED " for %s ignored",
+               name, extensions->disabled);
+    }
+    else if (extensions->stopped) {
+        jk_log(l, JK_LOG_WARNING,
+               "Worker %s is not of type lb, activation extension "
+               JK_UWMAP_EXTENSION_STOPPED " for %s ignored",
+               name, extensions->stopped);
+    }
+
+    JK_TRACE_EXIT(l);
+    return JK_TRUE;
+}
+
+static void extension_fix_session(jk_pool_t *p, const char *name, jk_worker_t *jw,
+                                  rule_extension_t *extensions, jk_logger_t *l)
+{
+    if (jw->type != JK_LB_WORKER_TYPE && extensions->session_cookie) {
+        jk_log(l, JK_LOG_WARNING,
+                "Worker %s is not of type lb, extension "
+                JK_UWMAP_EXTENSION_SESSION_COOKIE " for %s ignored",
+                name, extensions->session_cookie);
+    }
+    if (jw->type != JK_LB_WORKER_TYPE && extensions->session_path) {
+        jk_log(l, JK_LOG_WARNING,
+                "Worker %s is not of type lb, extension "
+                JK_UWMAP_EXTENSION_SESSION_PATH " for %s ignored",
+                name, extensions->session_path);
+    }
+}
+
+void extension_fix(jk_pool_t *p, const char *name,
+                   rule_extension_t *extensions, jk_logger_t *l)
+{
+    jk_worker_t *jw = wc_get_worker_for_name(name, l);
+    if(!jw) {
+        jk_log(l, JK_LOG_ERROR,
+               "Could not find worker with name '%s' in uri map post processing.",
+               name);
+        return;
+    }
+    if (!extension_fix_activation(p, name, jw, extensions, l))
+        return;
+    if (extensions->fail_on_status_str) {
+        extension_fix_fail_on_status(p, name, extensions, l);
+    }
+    extension_fix_session(p, name, jw, extensions, l);
+}
+
 void uri_worker_map_switch(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
 {
     int new_index;
@@ -498,77 +602,14 @@ void uri_worker_map_ext(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
 
     for (i = 0; i < IND_NEXT(uw_map->size); i++) {
         uri_worker_record_t *uwr = IND_NEXT(uw_map->maps)[i];
-        jk_worker_t *jw;
+        jk_pool_t *p;
         if(uwr->match_type & MATCH_TYPE_NO_MATCH)
             continue;
-        jw = wc_get_worker_for_name(uwr->worker_name, l);
-        if(!jw) {
-            jk_log(l, JK_LOG_ERROR,
-                   "Could not find worker with name '%s' in uri map post processing.",
-                   uwr->worker_name);
-            continue;
-        }
-        if (JK_IS_DEBUG_LEVEL(l))
-            jk_log(l, JK_LOG_DEBUG,
-                   "Checking extension for worker %d: %s of type %s (%d)",
-                   i, uwr->worker_name, wc_get_name_for_type(jw->type,l), jw->type);
-
-        if (jw->type == JK_LB_WORKER_TYPE &&
-            (uwr->extensions.active || uwr->extensions.disabled || uwr->extensions.stopped)) {
-            int j;
-            lb_worker_t *lb = (lb_worker_t *)jw->worker_private;
-            jk_pool_t *p;
-            if (!uwr->extensions.activation) {
-                uwr->extensions.activation_size = lb->num_of_workers;
-                if (uwr->source_type == SOURCE_TYPE_URIMAP)
-                    p = &IND_NEXT(uw_map->p_dyn);
-                else
-                    p = &uw_map->p;
-                uwr->extensions.activation = (int *)jk_pool_alloc(p,
-                                                        uwr->extensions.activation_size * sizeof(int));
-                if (!uwr->extensions.activation) {
-                    jk_log(l, JK_LOG_ERROR,
-                           "can't alloc extensions activation list");
-                    continue;
-                } else if (JK_IS_DEBUG_LEVEL(l))
-                    jk_log(l, JK_LOG_DEBUG,
-                           "Allocated activations array of size %d for lb worker %s",
-                           uwr->extensions.activation_size, uwr->worker_name);
-                for (j=0; j<uwr->extensions.activation_size; j++) {
-                    uwr->extensions.activation[j] = JK_LB_ACTIVATION_UNSET;
-                }
-            }
-            if (uwr->extensions.active)
-                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
-                                   uwr->extensions.active, JK_LB_ACTIVATION_ACTIVE, l);
-            if (uwr->extensions.disabled)
-                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
-                                   uwr->extensions.disabled, JK_LB_ACTIVATION_DISABLED, l);
-            if (uwr->extensions.stopped)
-                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
-                                   uwr->extensions.stopped, JK_LB_ACTIVATION_STOPPED, l);
-        }
-        else if (uwr->extensions.active) {
-            jk_log(l, JK_LOG_WARNING,
-                   "Worker %s is not of type lb, activation extension "
-                   JK_UWMAP_EXTENSION_ACTIVE " for %s ignored",
-                   uwr->worker_name, uwr->extensions.active);
-        }
-        else if (uwr->extensions.disabled) {
-            jk_log(l, JK_LOG_WARNING,
-                   "Worker %s is not of type lb, activation extension "
-                   JK_UWMAP_EXTENSION_DISABLED " for %s ignored",
-                   uwr->worker_name, uwr->extensions.disabled);
-        }
-        else if (uwr->extensions.stopped) {
-            jk_log(l, JK_LOG_WARNING,
-                   "Worker %s is not of type lb, activation extension "
-                   JK_UWMAP_EXTENSION_STOPPED " for %s ignored",
-                   uwr->worker_name, uwr->extensions.stopped);
-        }
-        if (uwr->extensions.fail_on_status_str) {
-            extract_fail_on_status(uw_map, uwr, l);
-        }
+        if (uwr->source_type == SOURCE_TYPE_URIMAP)
+            p = &IND_NEXT(uw_map->p_dyn);
+        else
+            p = &uw_map->p;
+        extension_fix(p, uwr->worker_name, &uwr->extensions, l);
     }
     if (JK_IS_DEBUG_LEVEL(l))
         uri_worker_map_dump(uw_map, "after extension stripping", l);
@@ -578,6 +619,133 @@ void uri_worker_map_ext(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
 
 }
 
+/* Parse rule extensions */
+void parse_rule_extensions(char *rule, rule_extension_t *extensions,
+                           jk_logger_t *l)
+{
+    char *param;
+#ifdef _MT_CODE_PTHREAD
+    char *lasts = NULL;
+#endif
+
+    extensions->reply_timeout = -1;
+    extensions->sticky_ignore = JK_FALSE;
+    extensions->stateless = JK_FALSE;
+    extensions->active = NULL;
+    extensions->disabled = NULL;
+    extensions->stopped = NULL;
+    extensions->activation_size = 0;
+    extensions->activation = NULL;
+    extensions->fail_on_status_size = 0;
+    extensions->fail_on_status = NULL;
+    extensions->fail_on_status_str = NULL;
+    extensions->use_server_error_pages = 0;
+    extensions->session_cookie = NULL;
+    extensions->session_path = NULL;
+
+#ifdef _MT_CODE_PTHREAD
+    param = strtok_r(rule, ";", &lasts);
+#else
+    param = strtok(rule, ";");
+#endif
+    if (param) {
+#ifdef _MT_CODE_PTHREAD
+        for (param = strtok_r(NULL, ";", &lasts); param; param = strtok_r(NULL, ";", &lasts)) {
+#else
+        for (param = strtok(NULL, ";"); param; param = strtok(NULL, ";")) {
+#endif
+            if (!strncmp(param, JK_UWMAP_EXTENSION_REPLY_TIMEOUT, strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT))) {
+                extensions->reply_timeout = atoi(param + strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT));
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_STICKY_IGNORE, strlen(JK_UWMAP_EXTENSION_STICKY_IGNORE))) {
+                int val = atoi(param + strlen(JK_UWMAP_EXTENSION_STICKY_IGNORE));
+                if (val) {
+                    extensions->sticky_ignore = JK_TRUE;
+                }
+                else {
+                    extensions->sticky_ignore = JK_FALSE;
+                }
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_STATELESS, strlen(JK_UWMAP_EXTENSION_STATELESS))) {
+                int val = atoi(param + strlen(JK_UWMAP_EXTENSION_STATELESS));
+                if (val) {
+                    extensions->stateless = JK_TRUE;
+                }
+                else {
+                    extensions->stateless = JK_FALSE;
+                }
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_USE_SRV_ERRORS, strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS))) {
+                extensions->use_server_error_pages = atoi(param + strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS));
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_ACTIVE, strlen(JK_UWMAP_EXTENSION_ACTIVE))) {
+                if (extensions->active)
+                    jk_log(l, JK_LOG_WARNING,
+                           "rule extension '%s' only allowed once",
+                           JK_UWMAP_EXTENSION_ACTIVE);
+                else
+                    extensions->active = param + strlen(JK_UWMAP_EXTENSION_ACTIVE);
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_DISABLED, strlen(JK_UWMAP_EXTENSION_DISABLED))) {
+                if (extensions->disabled)
+                    jk_log(l, JK_LOG_WARNING,
+                           "rule extension '%s' only allowed once",
+                           JK_UWMAP_EXTENSION_DISABLED);
+                else
+                    extensions->disabled = param + strlen(JK_UWMAP_EXTENSION_DISABLED);
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_STOPPED, strlen(JK_UWMAP_EXTENSION_STOPPED))) {
+                if (extensions->stopped)
+                    jk_log(l, JK_LOG_WARNING,
+                           "rule extension '%s' only allowed once",
+                           JK_UWMAP_EXTENSION_STOPPED);
+                else
+                    extensions->stopped = param + strlen(JK_UWMAP_EXTENSION_STOPPED);
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_FAIL_ON_STATUS, strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS))) {
+                if (extensions->fail_on_status_str)
+                    jk_log(l, JK_LOG_WARNING,
+                           "rule extension '%s' only allowed once",
+                           JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
+                else
+                    extensions->fail_on_status_str = param + strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_SESSION_COOKIE, strlen(JK_UWMAP_EXTENSION_SESSION_COOKIE))) {
+                if (extensions->session_cookie)
+                    jk_log(l, JK_LOG_WARNING,
+                           "extension '%s' in uri worker map only allowed once",
+                           JK_UWMAP_EXTENSION_SESSION_COOKIE);
+                else
+                    extensions->session_cookie = param + strlen(JK_UWMAP_EXTENSION_SESSION_COOKIE);
+            }
+            else if (!strncmp(param, JK_UWMAP_EXTENSION_SESSION_PATH, strlen(JK_UWMAP_EXTENSION_SESSION_PATH))) {
+                if (extensions->session_path)
+                    jk_log(l, JK_LOG_WARNING,
+                           "extension '%s' in uri worker map only allowed once",
+                           JK_UWMAP_EXTENSION_SESSION_PATH);
+                else {
+                    // Check if the session identifier starts with semicolon.
+                    if (!strcmp(param, JK_UWMAP_EXTENSION_SESSION_PATH)) {
+#ifdef _MT_CODE_PTHREAD
+                        param = strtok_r(NULL, ";", &lasts);
+#else
+                        param = strtok(NULL, ";");
+#endif
+                        extensions->session_path = param;
+                    } else
+                        extensions->session_path = param + strlen(JK_UWMAP_EXTENSION_SESSION_PATH);
+                }
+            }
+            else {
+                jk_log(l, JK_LOG_WARNING,
+                       "unknown rule extension '%s'",
+                       param);
+            }
+        }
+    }
+}
+
+
 /* Add new entry to NEXT generation */
 int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                        const char *puri, const char *worker,
@@ -629,81 +797,8 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
     }
 
     if (*uri == '/') {
-        char *w;
-        char *param;
-#ifdef _MT_CODE_PTHREAD
-        char *lasts = NULL;
-#endif
-
-        w = jk_pool_strdup(p, worker);
-        uwr->extensions.reply_timeout = -1;
-        uwr->extensions.active = NULL;
-        uwr->extensions.disabled = NULL;
-        uwr->extensions.stopped = NULL;
-        uwr->extensions.activation_size = 0;
-        uwr->extensions.activation = NULL;
-        uwr->extensions.fail_on_status_size = 0;
-        uwr->extensions.fail_on_status = NULL;
-        uwr->extensions.fail_on_status_str = NULL;
-        uwr->extensions.use_server_error_pages = 0;
-
-#ifdef _MT_CODE_PTHREAD
-        param = strtok_r(w, ";", &lasts);
-#else
-        param = strtok(w, ";");
-#endif
-        if (param) {
-#ifdef _MT_CODE_PTHREAD
-            for (param = strtok_r(NULL, ";", &lasts); param; param = strtok_r(NULL, ";", &lasts)) {
-#else
-            for (param = strtok(NULL, ";"); param; param = strtok(NULL, ";")) {
-#endif
-                if (!strncmp(param, JK_UWMAP_EXTENSION_REPLY_TIMEOUT, strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT))) {
-                    uwr->extensions.reply_timeout = atoi(param + strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT));
-                }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_USE_SRV_ERRORS, strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS))) {
-                    uwr->extensions.use_server_error_pages = atoi(param + strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS));
-                }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_ACTIVE, strlen(JK_UWMAP_EXTENSION_ACTIVE))) {
-                    if (uwr->extensions.active)
-                        jk_log(l, JK_LOG_WARNING,
-                               "extension '%s' in uri worker map only allowed once",
-                               JK_UWMAP_EXTENSION_ACTIVE);
-                    else
-                        uwr->extensions.active = param + strlen(JK_UWMAP_EXTENSION_ACTIVE);
-                }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_DISABLED, strlen(JK_UWMAP_EXTENSION_DISABLED))) {
-                    if (uwr->extensions.disabled)
-                        jk_log(l, JK_LOG_WARNING,
-                               "extension '%s' in uri worker map only allowed once",
-                               JK_UWMAP_EXTENSION_DISABLED);
-                    else
-                        uwr->extensions.disabled = param + strlen(JK_UWMAP_EXTENSION_DISABLED);
-                }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_STOPPED, strlen(JK_UWMAP_EXTENSION_STOPPED))) {
-                    if (uwr->extensions.stopped)
-                        jk_log(l, JK_LOG_WARNING,
-                               "extension '%s' in uri worker map only allowed once",
-                               JK_UWMAP_EXTENSION_STOPPED);
-                    else
-                        uwr->extensions.stopped = param + strlen(JK_UWMAP_EXTENSION_STOPPED);
-                }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_FAIL_ON_STATUS, strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS))) {
-                    if (uwr->extensions.fail_on_status_str)
-                        jk_log(l, JK_LOG_WARNING,
-                               "extension '%s' in uri worker map only allowed once",
-                               JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
-                    else
-                        uwr->extensions.fail_on_status_str = param + strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
-                }
-                else {
-                    jk_log(l, JK_LOG_WARNING,
-                           "unknown extension '%s' in uri worker map",
-                           param);
-                }
-            }
-        }
-
+        char *w = jk_pool_strdup(p, worker);
+        parse_rule_extensions(w, &uwr->extensions, l);
         uwr->source_type = source_type;
         uwr->worker_name = w;
         uwr->uri = uri;
@@ -992,7 +1087,7 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
             off = 1;
         }
         /* Size including leading slash. */
-        vhost_len = strlen(vhost);
+        vhost_len = (unsigned int)strlen(vhost);
         if (vhost_len + off >= JK_MAX_URI_LEN) {
             vhost_len = 0;
             jk_log(l, JK_LOG_WARNING,
@@ -1142,6 +1237,8 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
         if (JK_IS_DEBUG_LEVEL(l))
             uri_worker_map_dump(uw_map, "after file load", l);
         rc = JK_TRUE;
+    } else {
+        jk_log(l, JK_LOG_ERROR, "Failed to load uri_worker_map file %s (errno=%d, err=%s).", uw_map->fname, errno, strerror(errno));
     }
     jk_map_free(&map);
     return rc;
@@ -1170,10 +1267,10 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
                        uw_map->fname);
             return JK_TRUE;
         }
-        JK_ENTER_CS(&(uw_map->cs), rc);
+        JK_ENTER_CS(&uw_map->cs);
         /* Check if some other thread updated status */
         if (statbuf.st_mtime == uw_map->modified) {
-            JK_LEAVE_CS(&(uw_map->cs), rc);
+            JK_LEAVE_CS(&uw_map->cs);
             if (JK_IS_DEBUG_LEVEL(l))
                 jk_log(l, JK_LOG_DEBUG,
                        "File %s  is not modified",
@@ -1183,7 +1280,7 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
         rc = uri_worker_map_load(uw_map, l);
         uri_worker_map_ext(uw_map, l);
         uri_worker_map_switch(uw_map, l);
-        JK_LEAVE_CS(&(uw_map->cs), rc);
+        JK_LEAVE_CS(&uw_map->cs);
         jk_log(l, JK_LOG_INFO,
                "Reloaded urimaps from %s", uw_map->fname);
     }
