@@ -20,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 1078846 $                                          *
+ * Version:     $Revision: 1338797 $                                          *
  ***************************************************************************/
 
 
@@ -31,99 +31,99 @@
 #include "jk_lb_worker.h"
 #include "jk_mt.h"
 
-#define SYSPROPS_OF_WORKER          ("sysprops")
-#define STDERR_OF_WORKER            ("stderr")
-#define STDOUT_OF_WORKER            ("stdout")
-#define SECRET_OF_WORKER            ("secret")
-#define MX_OF_WORKER                ("mx")
-#define MS_OF_WORKER                ("ms")
-#define CP_OF_WORKER                ("class_path")
-#define BRIDGE_OF_WORKER            ("bridge")
-#define JVM_OF_WORKER               ("jvm_lib")
-#define LIBPATH_OF_WORKER           ("ld_path")
-#define CMD_LINE_OF_WORKER          ("cmd_line")
-#define NATIVE_LIB_OF_WORKER        ("native_lib")
-#define REFERENCE_OF_WORKER         ("reference")
-#define HOST_OF_WORKER              ("host")
-#define PORT_OF_WORKER              ("port")
-#define TYPE_OF_WORKER              ("type")
-#define CACHE_OF_WORKER_DEPRECATED  ("cachesize")
-#define CACHE_OF_WORKER             ("connection_pool_size")
-#define CACHE_OF_WORKER_MIN         ("connection_pool_minsize")
-#define CACHE_TIMEOUT_DEPRECATED    ("cache_timeout")
-#define CACHE_TIMEOUT_OF_WORKER     ("connection_pool_timeout")
-#define CACHE_ACQUIRE_OF_WORKER     ("connection_acquire_timeout")
-#define RECOVERY_OPTS_OF_WORKER     ("recovery_options")
-#define CONNECT_TIMEOUT_OF_WORKER   ("connect_timeout")
-#define PREPOST_TIMEOUT_OF_WORKER   ("prepost_timeout")
-#define REPLY_TIMEOUT_OF_WORKER     ("reply_timeout")
-#define SOCKET_TIMEOUT_OF_WORKER    ("socket_timeout")
-#define SOCKET_CONNECT_TIMEOUT_OF_WORKER ("socket_connect_timeout")
-#define PING_TIMEOUT_OF_WORKER      ("ping_timeout")
-#define PING_MODE_OF_WORKER         ("ping_mode")
-#define SOCKET_BUFFER_OF_WORKER     ("socket_buffer")
-#define SOCKET_KEEPALIVE_OF_WORKER  ("socket_keepalive")
-#define CONN_PING_INTERVAL_OF_WORKER  ("connection_ping_interval")
-#define RECYCLE_TIMEOUT_DEPRECATED  ("recycle_timeout")
-#define LOAD_FACTOR_OF_WORKER       ("lbfactor")
-#define DISTANCE_OF_WORKER          ("distance")
-#define BALANCED_WORKERS_DEPRECATED ("balanced_workers")
-#define BALANCE_WORKERS             ("balance_workers")
-#define STICKY_SESSION              ("sticky_session")
-#define STICKY_SESSION_FORCE        ("sticky_session_force")
-#define SESSION_COOKIE_OF_WORKER    ("session_cookie")
-#define SESSION_PATH_OF_WORKER      ("session_path")
+#define SYSPROPS_OF_WORKER          "sysprops"
+#define STDERR_OF_WORKER            "stderr"
+#define STDOUT_OF_WORKER            "stdout"
+#define SECRET_OF_WORKER            "secret"
+#define MX_OF_WORKER                "mx"
+#define MS_OF_WORKER                "ms"
+#define CP_OF_WORKER                "class_path"
+#define BRIDGE_OF_WORKER            "bridge"
+#define JVM_OF_WORKER               "jvm_lib"
+#define LIBPATH_OF_WORKER           "ld_path"
+#define CMD_LINE_OF_WORKER          "cmd_line"
+#define NATIVE_LIB_OF_WORKER        "native_lib"
+#define REFERENCE_OF_WORKER         "reference"
+#define HOST_OF_WORKER              "host"
+#define PORT_OF_WORKER              "port"
+#define TYPE_OF_WORKER              "type"
+#define CACHE_OF_WORKER_DEPRECATED  "cachesize"
+#define CACHE_OF_WORKER             "connection_pool_size"
+#define CACHE_OF_WORKER_MIN         "connection_pool_minsize"
+#define CACHE_TIMEOUT_DEPRECATED    "cache_timeout"
+#define CACHE_TIMEOUT_OF_WORKER     "connection_pool_timeout"
+#define CACHE_ACQUIRE_OF_WORKER     "connection_acquire_timeout"
+#define RECOVERY_OPTS_OF_WORKER     "recovery_options"
+#define CONNECT_TIMEOUT_OF_WORKER   "connect_timeout"
+#define PREPOST_TIMEOUT_OF_WORKER   "prepost_timeout"
+#define REPLY_TIMEOUT_OF_WORKER     "reply_timeout"
+#define SOCKET_TIMEOUT_OF_WORKER    "socket_timeout"
+#define SOCKET_CONNECT_TIMEOUT_OF_WORKER "socket_connect_timeout"
+#define PING_TIMEOUT_OF_WORKER      "ping_timeout"
+#define PING_MODE_OF_WORKER         "ping_mode"
+#define SOCKET_BUFFER_OF_WORKER     "socket_buffer"
+#define SOCKET_KEEPALIVE_OF_WORKER  "socket_keepalive"
+#define CONN_PING_INTERVAL_OF_WORKER  "connection_ping_interval"
+#define RECYCLE_TIMEOUT_DEPRECATED  "recycle_timeout"
+#define LOAD_FACTOR_OF_WORKER       "lbfactor"
+#define DISTANCE_OF_WORKER          "distance"
+#define BALANCED_WORKERS_DEPRECATED "balanced_workers"
+#define BALANCE_WORKERS             "balance_workers"
+#define STICKY_SESSION              "sticky_session"
+#define STICKY_SESSION_FORCE        "sticky_session_force"
+#define SESSION_COOKIE_OF_WORKER    "session_cookie"
+#define SESSION_PATH_OF_WORKER      "session_path"
 
-#define LOCAL_WORKER_DEPRECATED     ("local_worker")
-#define LOCAL_WORKER_ONLY_DEPRECATED ("local_worker_only")
-#define JVM_ROUTE_OF_WORKER_DEPRECATED ("jvm_route")
-#define ROUTE_OF_WORKER             ("route")
-#define DOMAIN_OF_WORKER            ("domain")
-#define REDIRECT_OF_WORKER          ("redirect")
-#define MOUNT_OF_WORKER             ("mount")
-#define METHOD_OF_WORKER            ("method")
-#define LOCK_OF_WORKER              ("lock")
-#define IS_WORKER_DISABLED_DEPRECATED ("disabled")
-#define IS_WORKER_STOPPED_DEPRECATED  ("stopped")
-#define ACTIVATION_OF_WORKER        ("activation")
-#define WORKER_RECOVER_TIME         ("recover_time")
-#define WORKER_ERROR_ESCALATION_TIME ("error_escalation_time")
-#define MAX_REPLY_TIMEOUTS_OF_WORKER ("max_reply_timeouts")
-#define RETRY_INTERVAL_OF_WORKER    ("retry_interval")
-#define WORKER_MAX_PACKET_SIZE      ("max_packet_size")
-#define STYLE_SHEET_OF_WORKER       ("css")
-#define NAMESPACE_OF_WORKER         ("ns")
-#define XML_NAMESPACE_OF_WORKER     ("xmlns")
-#define XML_DOCTYPE_OF_WORKER       ("doctype")
-#define PROP_PREFIX_OF_WORKER       ("prefix")
+#define LOCAL_WORKER_DEPRECATED     "local_worker"
+#define LOCAL_WORKER_ONLY_DEPRECATED "local_worker_only"
+#define JVM_ROUTE_OF_WORKER_DEPRECATED "jvm_route"
+#define ROUTE_OF_WORKER             "route"
+#define DOMAIN_OF_WORKER            "domain"
+#define REDIRECT_OF_WORKER          "redirect"
+#define MOUNT_OF_WORKER             "mount"
+#define METHOD_OF_WORKER            "method"
+#define LOCK_OF_WORKER              "lock"
+#define IS_WORKER_DISABLED_DEPRECATED "disabled"
+#define IS_WORKER_STOPPED_DEPRECATED  "stopped"
+#define ACTIVATION_OF_WORKER        "activation"
+#define WORKER_RECOVER_TIME         "recover_time"
+#define WORKER_ERROR_ESCALATION_TIME "error_escalation_time"
+#define MAX_REPLY_TIMEOUTS_OF_WORKER "max_reply_timeouts"
+#define RETRY_INTERVAL_OF_WORKER    "retry_interval"
+#define WORKER_MAX_PACKET_SIZE      "max_packet_size"
+#define STYLE_SHEET_OF_WORKER       "css"
+#define NAMESPACE_OF_WORKER         "ns"
+#define XML_NAMESPACE_OF_WORKER     "xmlns"
+#define XML_DOCTYPE_OF_WORKER       "doctype"
+#define PROP_PREFIX_OF_WORKER       "prefix"
 
-#define READ_ONLY_OF_WORKER         ("read_only")
-#define USER_OF_WORKER              ("user")
-#define USER_CASE_OF_WORKER         ("user_case_insensitive")
-#define GOOD_RATING_OF_WORKER       ("good")
-#define BAD_RATING_OF_WORKER        ("bad")
+#define READ_ONLY_OF_WORKER         "read_only"
+#define USER_OF_WORKER              "user"
+#define USER_CASE_OF_WORKER         "user_case_insensitive"
+#define GOOD_RATING_OF_WORKER       "good"
+#define BAD_RATING_OF_WORKER        "bad"
 
 #define DEFAULT_WORKER_TYPE         JK_AJP13_WORKER_NAME
-#define SECRET_KEY_OF_WORKER        ("secretkey")
-#define RETRIES_OF_WORKER           ("retries")
-#define STATUS_FAIL_OF_WORKER       ("fail_on_status")
+#define SECRET_KEY_OF_WORKER        "secretkey"
+#define RETRIES_OF_WORKER           "retries"
+#define STATUS_FAIL_OF_WORKER       "fail_on_status"
 
 #define DEFAULT_WORKER              JK_AJP13_WORKER_NAME
-#define WORKER_LIST_PROPERTY_NAME     ("worker.list")
-#define LIST_PROPERTY_NAME            ("list")
-#define WORKER_MAINTAIN_PROPERTY_NAME ("worker.maintain")
-#define MAINTAIN_PROPERTY_NAME        ("maintain")
-#define DEFAULT_MAINTAIN_TIME       (60)
-#define DEFAULT_LB_FACTOR           (1)
-#define DEFAULT_DISTANCE            (0)
+#define WORKER_LIST_PROPERTY_NAME     "worker.list"
+#define LIST_PROPERTY_NAME            "list"
+#define WORKER_MAINTAIN_PROPERTY_NAME "worker.maintain"
+#define MAINTAIN_PROPERTY_NAME        "maintain"
+#define DEFAULT_MAINTAIN_TIME       60
+#define DEFAULT_LB_FACTOR           1
+#define DEFAULT_DISTANCE            0
 
-#define TOMCAT32_BRIDGE_NAME        ("tomcat32")
-#define TOMCAT33_BRIDGE_NAME        ("tomcat33")
-#define TOMCAT40_BRIDGE_NAME        ("tomcat40")
-#define TOMCAT41_BRIDGE_NAME        ("tomcat41")
-#define TOMCAT50_BRIDGE_NAME        ("tomcat5")
+#define TOMCAT32_BRIDGE_NAME        "tomcat32"
+#define TOMCAT33_BRIDGE_NAME        "tomcat33"
+#define TOMCAT40_BRIDGE_NAME        "tomcat40"
+#define TOMCAT41_BRIDGE_NAME        "tomcat41"
+#define TOMCAT50_BRIDGE_NAME        "tomcat5"
 
-#define HUGE_BUFFER_SIZE (8*1024)
+#define HUGE_BUFFER_SIZE 8192
 
 #define MAKE_WORKER_PARAM(P)     \
         strcpy(buf, "worker.");  \
@@ -144,9 +144,9 @@
 #define JK_TIME_FORMAT_NONE   "[%a %b %d %H:%M:%S %Y] "
 #define JK_TIME_FORMAT_MILLI  "[%a %b %d %H:%M:%S." JK_TIME_CONV_MILLI " %Y] "
 #define JK_TIME_FORMAT_MICRO  "[%a %b %d %H:%M:%S." JK_TIME_CONV_MICRO " %Y] "
-#define JK_TIME_SUBSEC_NONE   (0)
-#define JK_TIME_SUBSEC_MILLI  (1)
-#define JK_TIME_SUBSEC_MICRO  (2)
+#define JK_TIME_SUBSEC_NONE   0
+#define JK_TIME_SUBSEC_MILLI  1
+#define JK_TIME_SUBSEC_MICRO  2
 
 /* Visual C++ Toolkit 2003 support */
 #if defined (_MSC_VER) && (_MSC_VER == 1310)
@@ -347,13 +347,13 @@ int jk_get_bool_code(const char *v, int def)
     if (!v) {
         return def;
     }
-    else if (!strcasecmp(v, "off") ||
+    if (!strcasecmp(v, "off") ||
              *v == 'F' || *v == 'f' ||
              *v == 'N' || *v == 'n' ||
             (*v == '0' && *(v + 1) == '\0')) {
         return 0;
     }
-    else if (!strcasecmp(v, "on") ||
+    if (!strcasecmp(v, "on") ||
              *v == 'T' || *v == 't' ||
              *v == 'Y' || *v == 'y' ||
             (*v == '1' && *(v + 1) == '\0')) {
@@ -408,15 +408,16 @@ void jk_set_time_fmt(jk_logger_t *l, const char *jk_log_fmt)
         l->log_fmt_size = 0;
         l->log_fmt = jk_log_fmt;
 
-/* Look for the first occurence of JK_TIME_CONV_MILLI */
+        /* Look for the first occurence of JK_TIME_CONV_MILLI */
         if ((s = strstr(jk_log_fmt, JK_TIME_CONV_MILLI))) {
             size_t offset = s - jk_log_fmt;
             size_t len = strlen(JK_TIME_PATTERN_MILLI);
 
-/* If we don't have enough space in our fixed-length char array,
- * we simply stick to the default format, ignoring JK_TIME_CONV_MILLI.
- * Otherwise we replace the first occurence of JK_TIME_CONV_MILLI by JK_TIME_PATTERN_MILLI.
- */
+            /* If we don't have enough space in our fixed-length char array,
+             * we simply stick to the default format, ignoring JK_TIME_CONV_MILLI.
+             * Otherwise we replace the first occurence of
+             * JK_TIME_CONV_MILLI by JK_TIME_PATTERN_MILLI.
+             */
             if (offset + len < JK_TIME_MAX_SIZE) {
                 l->log_fmt_type = JK_TIME_SUBSEC_MILLI;
                 l->log_fmt_offset = offset;
@@ -425,22 +426,23 @@ void jk_set_time_fmt(jk_logger_t *l, const char *jk_log_fmt)
                 strncpy(l->log_fmt_subsec + offset + len,
                         s + strlen(JK_TIME_CONV_MILLI),
                         JK_TIME_MAX_SIZE - offset - len - 1);
-/* Now we put a stop mark into the string to make it's length at most JK_TIME_MAX_SIZE-1
- * plus terminating '\0'.
- */
+                /* Now we put a stop mark into the string to make it's length
+                 * at most JK_TIME_MAX_SIZE-1 plus terminating '\0'.
+                 */
                 l->log_fmt_subsec[JK_TIME_MAX_SIZE-1] = '\0';
                 l->log_fmt_size = strlen(l->log_fmt_subsec);
             }
-/* Look for the first occurence of JK_TIME_CONV_MICRO */
+            /* Look for the first occurence of JK_TIME_CONV_MICRO */
         }
         else if ((s = strstr(jk_log_fmt, JK_TIME_CONV_MICRO))) {
             size_t offset = s - jk_log_fmt;
             size_t len = strlen(JK_TIME_PATTERN_MICRO);
 
-/* If we don't have enough space in our fixed-length char array,
- * we simply stick to the default format, ignoring JK_TIME_CONV_MICRO.
- * Otherwise we replace the first occurence of JK_TIME_CONV_MICRO by JK_TIME_PATTERN_MICRO.
- */
+            /* If we don't have enough space in our fixed-length char array,
+             * we simply stick to the default format, ignoring JK_TIME_CONV_MICRO.
+             * Otherwise we replace the first occurence of JK_TIME_CONV_MICRO
+             * by JK_TIME_PATTERN_MICRO.
+             */
             if (offset + len < JK_TIME_MAX_SIZE) {
                 l->log_fmt_type = JK_TIME_SUBSEC_MICRO;
                 l->log_fmt_offset = offset;
@@ -449,9 +451,9 @@ void jk_set_time_fmt(jk_logger_t *l, const char *jk_log_fmt)
                 strncpy(l->log_fmt_subsec + offset + len,
                         s + strlen(JK_TIME_CONV_MICRO),
                         JK_TIME_MAX_SIZE - offset - len - 1);
-/* Now we put a stop mark into the string to make it's length at most JK_TIME_MAX_SIZE-1
- * plus terminating '\0'.
- */
+                /* Now we put a stop mark into the string to make it's length
+                 * at most JK_TIME_MAX_SIZE-1 plus terminating '\0'.
+                 */
                 l->log_fmt_subsec[JK_TIME_MAX_SIZE-1] = '\0';
                 l->log_fmt_size = strlen(l->log_fmt_subsec);
             }
@@ -469,11 +471,11 @@ static int set_time_str(char *str, int len, jk_logger_t *l)
     struct tm res;
 #endif
     int done;
-/* We want to use a fixed maximum size buffer here.
- * If we would dynamically adjust it to the real format
- * string length, we could support longer format strings,
- * but we would have to allocate and free for each log line.
- */
+    /* We want to use a fixed maximum size buffer here.
+     * If we would dynamically adjust it to the real format
+     * string length, we could support longer format strings,
+     * but we would have to allocate and free for each log line.
+     */
     char log_fmt[JK_TIME_MAX_SIZE];
 
     if (!l || !l->log_fmt) {
@@ -493,11 +495,11 @@ static int set_time_str(char *str, int len, jk_logger_t *l)
         rc = gettimeofday(&tv, NULL);
 #endif
         if (rc == 0) {
-/* We need this subsec buffer, because we convert
- * the integer with sprintf(), but we don't
- * want to write the terminating '\0' into our
- * final log format string.
- */
+            /* We need this subsec buffer, because we convert
+             * the integer with sprintf(), but we don't
+             * want to write the terminating '\0' into our
+             * final log format string.
+             */
             char subsec[7];
             t = tv.tv_sec;
             strncpy(log_fmt, l->log_fmt_subsec, l->log_fmt_size + 1);
@@ -565,30 +567,18 @@ static int JK_METHOD log_to_file(jk_logger_t *l, int level, int used, char *what
 
 int jk_parse_log_level(const char *level)
 {
-    if (0 == strcasecmp(level, JK_LOG_TRACE_VERB)) {
+    if (!strcasecmp(level, JK_LOG_TRACE_VERB))
         return JK_LOG_TRACE_LEVEL;
-    }
-
-    if (0 == strcasecmp(level, JK_LOG_DEBUG_VERB)) {
+    if (!strcasecmp(level, JK_LOG_DEBUG_VERB))
         return JK_LOG_DEBUG_LEVEL;
-    }
-
-    if (0 == strcasecmp(level, JK_LOG_INFO_VERB)) {
+    if (!strcasecmp(level, JK_LOG_INFO_VERB))
         return JK_LOG_INFO_LEVEL;
-    }
-
-    if (0 == strcasecmp(level, JK_LOG_WARN_VERB)) {
+    if (!strcasecmp(level, JK_LOG_WARN_VERB))
         return JK_LOG_WARNING_LEVEL;
-    }
-
-    if (0 == strcasecmp(level, JK_LOG_ERROR_VERB)) {
+    if (!strcasecmp(level, JK_LOG_ERROR_VERB))
         return JK_LOG_ERROR_LEVEL;
-    }
-
-    if (0 == strcasecmp(level, JK_LOG_EMERG_VERB)) {
+    if (!strcasecmp(level, JK_LOG_EMERG_VERB))
         return JK_LOG_EMERG_LEVEL;
-    }
-
     return JK_LOG_DEF_LEVEL;
 }
 
@@ -721,10 +711,11 @@ int jk_log(jk_logger_t *l,
         used = set_time_str(buf, usable_size, l);
 
         if (line) { /* line==0 only used for request log item */
-            /* Log [pid:threadid] for all levels except REQUEST. */
-            /* This information helps to correlate lines from different logs. */
-            /* Performance is no issue, because with production log levels */
-            /* we only call it often, if we have a lot of errors */
+            /* Log [pid:threadid] for all levels except REQUEST.
+             * This information helps to correlate lines from different logs.
+             * Performance is no issue, because with production log levels
+             * we only call it often, if we have a lot of errors
+             */
             rc = snprintf(buf + used, usable_size - used,
                           "[%" JK_PID_T_FMT ":%" JK_PTHREAD_T_FMT "] ", getpid(), jk_gettid());
             used += rc;
@@ -776,7 +767,8 @@ int jk_log(jk_logger_t *l,
         va_end(args);
         if ( rc <= usable_size - used ) {
             used += rc;
-        } else {
+        }
+        else {
             used = usable_size;
         }
         l->log(l, level, used, buf);
@@ -1242,6 +1234,7 @@ int jk_get_is_worker_disabled(jk_map_t *m, const char *wname)
 {
     int rc = JK_TRUE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(IS_WORKER_DISABLED_DEPRECATED);
@@ -1256,6 +1249,7 @@ int jk_get_is_worker_stopped(jk_map_t *m, const char *wname)
 {
     int rc = JK_TRUE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(IS_WORKER_STOPPED_DEPRECATED);
@@ -1270,21 +1264,20 @@ int jk_get_worker_activation(jk_map_t *m, const char *wname)
 {
     char buf[1024];
     const char *v;
+
     if (!m || !wname) {
         return JK_LB_ACTIVATION_ACTIVE;
     }
 
     MAKE_WORKER_PARAM(ACTIVATION_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
-    if (v) {
+    if (v)
         return jk_lb_get_activation_code(v);
-    }
-    else if (jk_get_is_worker_stopped(m, wname))
+    if (jk_get_is_worker_stopped(m, wname))
         return JK_LB_ACTIVATION_STOPPED;
-    else if (jk_get_is_worker_disabled(m, wname))
+    if (jk_get_is_worker_disabled(m, wname))
         return JK_LB_ACTIVATION_DISABLED;
-    else
-        return JK_LB_ACTIVATION_DEF;
+    return JK_LB_ACTIVATION_DEF;
 }
 
 int jk_get_lb_factor(jk_map_t *m, const char *wname)
@@ -1317,6 +1310,7 @@ int jk_get_is_sticky_session(jk_map_t *m, const char *wname)
 {
     int rc = JK_TRUE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(STICKY_SESSION);
@@ -1331,6 +1325,7 @@ int jk_get_is_sticky_session_force(jk_map_t *m, const char *wname)
 {
     int rc = JK_FALSE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(STICKY_SESSION_FORCE);
@@ -1345,6 +1340,7 @@ int jk_get_lb_method(jk_map_t *m, const char *wname)
 {
     char buf[1024];
     const char *v;
+
     if (!m || !wname) {
         return JK_LB_METHOD_DEF;
     }
@@ -1358,6 +1354,7 @@ int jk_get_lb_lock(jk_map_t *m, const char *wname)
 {
     char buf[1024];
     const char *v;
+
     if (!m || !wname) {
         return JK_LB_LOCK_DEF;
     }
@@ -1391,6 +1388,7 @@ int jk_get_worker_fail_on_status(jk_map_t *m, const char *wname,
                                  int *list, unsigned int list_size)
 {
     char buf[1024];
+
     if (!m || !wname || !list) {
         return 0;
     }
@@ -1408,6 +1406,7 @@ int jk_get_worker_user_case_insensitive(jk_map_t *m, const char *wname)
 {
     int rc = JK_FALSE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(USER_CASE_OF_WORKER);
@@ -1436,6 +1435,7 @@ const char *jk_get_worker_name_space(jk_map_t *m, const char *wname, const char
 {
     const char *rc;
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1451,6 +1451,7 @@ const char *jk_get_worker_xmlns(jk_map_t *m, const char *wname, const char *def)
 {
     const char *rc;
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1465,6 +1466,7 @@ const char *jk_get_worker_xmlns(jk_map_t *m, const char *wname, const char *def)
 const char *jk_get_worker_xml_doctype(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1475,6 +1477,7 @@ const char *jk_get_worker_xml_doctype(jk_map_t *m, const char *wname, const char
 const char *jk_get_worker_prop_prefix(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1486,6 +1489,7 @@ int jk_get_is_read_only(jk_map_t *m, const char *wname)
 {
     int rc = JK_FALSE;
     char buf[1024];
+
     if (m && wname) {
         int value;
         MAKE_WORKER_PARAM(READ_ONLY_OF_WORKER);
@@ -1777,7 +1781,7 @@ int jk_file_exists(const char *f)
         struct stat st;
 
         if ((0 == jk_stat(f, &st)) && (st.st_mode & S_IFREG))
-      return JK_TRUE;
+            return JK_TRUE;
     }
 
     return JK_FALSE;
@@ -1796,7 +1800,7 @@ static int jk_is_some_property(const char *prp_name, const char *suffix, const c
         suffix_len = strlen(buf);
         if (prp_name_len >= suffix_len) {
             const char *prp_suffix = prp_name + prp_name_len - suffix_len;
-            if (0 == strcmp(buf, prp_suffix)) {
+            if (!strcmp(buf, prp_suffix)) {
                 return JK_TRUE;
             }
         }
@@ -1818,6 +1822,7 @@ int jk_is_cmd_line_property(const char *prp_name)
 int jk_is_list_property(const char *prp_name)
 {
     const char **props = &list_properties[0];
+
     while (*props) {
         if (jk_is_some_property(prp_name, *props, "."))
             return JK_TRUE;
@@ -1829,6 +1834,7 @@ int jk_is_list_property(const char *prp_name)
 int jk_is_unique_property(const char *prp_name)
 {
     const char **props = &unique_properties[0];
+
     while (*props) {
         if (jk_is_some_property(prp_name, *props, "."))
             return JK_TRUE;
@@ -1840,6 +1846,7 @@ int jk_is_unique_property(const char *prp_name)
 int jk_is_deprecated_property(const char *prp_name)
 {
     const char **props = &deprecated_properties[0];
+
     while (*props) {
         if (jk_is_some_property(prp_name, *props, "."))
             return JK_TRUE;
@@ -1854,6 +1861,7 @@ int jk_is_deprecated_property(const char *prp_name)
 int jk_is_valid_property(const char *prp_name)
 {
     const char **props;
+
     if (memcmp(prp_name, "worker.", 7))
         return JK_TRUE;
 
@@ -1933,6 +1941,7 @@ int jk_get_worker_libpath(jk_map_t *m, const char *wname, const char **libpath)
 const char *jk_get_lb_session_cookie(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1943,6 +1952,7 @@ const char *jk_get_lb_session_cookie(jk_map_t *m, const char *wname, const char
 const char *jk_get_lb_session_path(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
+
     if (!m || !wname) {
         return NULL;
     }
@@ -1956,10 +1966,11 @@ int is_http_status_fail(unsigned int http_status_fail_num,
 {
     unsigned int i;
     int soft_status = -1 * status;
+
     for (i = 0; i < http_status_fail_num; i++) {
         if (http_status_fail[i] == status)
             return 1;
-        else if (http_status_fail[i] == soft_status)
+        if (http_status_fail[i] == soft_status)
             return -1;
     }
     return 0;
@@ -2037,62 +2048,14 @@ void jk_append_libpath(jk_pool_t *p, const char *libpath)
 
 void jk_init_ws_service(jk_ws_service_t *s)
 {
-    s->ws_private = NULL;
-    s->pool = NULL;
-    s->method = NULL;
-    s->protocol = NULL;
-    s->req_uri = NULL;
-    s->remote_addr = NULL;
-    s->remote_port = NULL;
-    s->remote_host = NULL;
-    s->remote_user = NULL;
-    s->auth_type = NULL;
-    s->query_string = NULL;
-    s->server_name = NULL;
-    s->server_port = 80;
-    s->server_software = NULL;
-    s->content_length = 0;
-    s->is_chunked = 0;
-    s->no_more_chunks = 0;
-    s->content_read = 0;
-    s->is_ssl = JK_FALSE;
-    s->ssl_cert = NULL;
-    s->ssl_cert_len = 0;
-    s->ssl_cipher = NULL;
-    s->ssl_session = NULL;
+    memset(s, 0, sizeof(jk_ws_service_t));
+
+    s->server_port  = 80;
     s->ssl_key_size = -1;
-    s->headers_names = NULL;
-    s->headers_values = NULL;
-    s->num_headers = 0;
-    s->attributes_names = NULL;
-    s->attributes_values = NULL;
-    s->num_attributes = 0;
-    s->route = NULL;
-    s->activation = JK_LB_ACTIVATION_ACTIVE;
-    s->secret = NULL;
-    s->reco_buf = NULL;
-    s->reco_status = RECO_NONE;
-    s->flush_packets = JK_FALSE;
-    s->flush_header = JK_FALSE;
-    s->extension.reply_timeout = -1;
-    s->extension.use_server_error_pages = 0;
-    s->extension.activation = NULL;
-    s->extension.fail_on_status_size = 0;
-    s->extension.fail_on_status = NULL;
-    s->response_started = JK_FALSE;
-    s->response_blocked = JK_FALSE;
-    s->http_response_status = JK_HTTP_OK;
-    s->uw_map = NULL;
-    s->start_response = NULL;
-    s->read = NULL;
-    s->write = NULL;
-    s->flush = NULL;
-    s->done = NULL;
-    s->disable_reuse = JK_FALSE;
-    s->add_log_items = NULL;
-    s->next_vhost = NULL;
-    s->vhost_to_text = NULL;
-    s->vhost_to_uw_map = NULL;
+    s->activation   = JK_LB_ACTIVATION_TEXT_ACTIVE;
+    s->reco_status  = RECO_NONE;
+    s->extension.reply_timeout  = -1;
+    s->http_response_status     = JK_HTTP_OK;
 }
 
 /* Match = 0, NoMatch = 1, Abort = -1
@@ -2251,3 +2214,62 @@ void jk_ebcdic2ascii(char *src, char *dst) {
 }
 
 #endif
+
+#if defined (WIN32)
+
+static PSECURITY_ATTRIBUTES pNullSA;
+static SECURITY_ATTRIBUTES  stEmptySA;
+/* To share the objects with other processes, we need a 0 ACL
+ * Code from MS KB Q106387
+ */
+PSECURITY_ATTRIBUTES jk_get_sa_with_null_dacl()
+{
+    DWORD rc = 0;
+    PSECURITY_DESCRIPTOR pSD;
+
+    if (pNullSA != NULL) {
+        return pNullSA;
+    }
+    else {
+        stEmptySA.nLength = (DWORD)sizeof(SECURITY_ATTRIBUTES);
+        stEmptySA.lpSecurityDescriptor = 0;
+    }
+
+    if (!(pNullSA = LocalAlloc(LPTR, sizeof(SECURITY_ATTRIBUTES)))) {
+        rc = GetLastError();
+        goto cleanup;
+    }
+    pNullSA->nLength = (DWORD)sizeof(SECURITY_ATTRIBUTES);
+    pSD = LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
+    if (pSD == 0) {
+        rc = GetLastError();
+        goto cleanup;
+    }
+    pNullSA->lpSecurityDescriptor = pSD;
+    if (!InitializeSecurityDescriptor(pSD, SECURITY_DESCRIPTOR_REVISION)) {
+        rc = GetLastError();
+        goto cleanup;
+    }
+    if (!SetSecurityDescriptorDacl(pSD, TRUE, (PACL)0, FALSE)) {
+        rc = GetLastError();
+        goto cleanup;
+    }
+    pNullSA->lpSecurityDescriptor = pSD;
+    pNullSA->bInheritHandle       = FALSE;
+
+    SetLastError(0);
+    return pNullSA;
+
+cleanup:
+    if (pSD)
+        LocalFree(pSD);
+    if (pNullSA)
+        LocalFree(pNullSA);
+
+    pNullSA = &stEmptySA;
+    pNullSA->bInheritHandle = FALSE;
+
+    SetLastError(rc);
+    return pNullSA;
+}
+#endif
