@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: NSAPI plugin for Netscape servers                          *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 1078805 $                                           *
+ * Version:     $Revision: 1306209 $                                           *
  ***************************************************************************/
 
 #if defined(WIN32)
@@ -65,7 +65,7 @@ static jk_logger_t *logger = NULL;
 static jk_worker_env_t worker_env;
 static jk_map_t *init_map = NULL;
 static jk_uri_worker_map_t *uw_map = NULL;
-static size_t jk_shm_size = 0;
+static int jk_shm_size = 0;
 
 #ifdef NETWARE
 int (*PR_IsSocketSecure) (SYS_NETFD * csd);     /* pointer to PR_IsSocketSecure function */
@@ -378,7 +378,7 @@ NSAPI_PUBLIC void jk_term(void *p)
     }
 
     wc_close(logger);
-    jk_shm_close();
+    jk_shm_close(logger);
     if (logger) {
         jk_close_file_logger(&logger);
     }
