@@ -1,5 +1,5 @@
-#!/bin/ksh
-
+#!/bin/bash
+#
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -33,8 +33,7 @@ JK_OWNER="root"
 JK_GROUP="bin"
 JK_TOOLS="`pwd`"
 
-COPY_TOP="KEYS"
-COPY_JK="BUILD.txt native jkstatus support tools xdocs"
+COPY_JK="README.txt HOWTO-RELEASE.txt native jkstatus support tools xdocs"
 COPY_NATIVE="LICENSE NOTICE"
 COPY_BUILD="docs"
 COPY_CONF="httpd-jk.conf uriworkermap.properties workers.properties workers.properties.minimal"
@@ -45,13 +44,19 @@ SIGN_OPTS=""
 #################### FUNCTIONS ##############
 
 usage() {
-    echo "Usage:: $0 -t VERSION [-r revision] [-b BRANCH | -T | -d DIR]"
-    echo "        -t: version to package"
+    echo "Usage:: $0 -v VERSION [-f] [-r revision] [-t tag | -b BRANCH | -T | -d DIR]"
+    echo "        -v: version to package"
+    echo "        -f: force, do not validate tag against version"
+    echo "        -h: create text documentation for html"
+    echo "        -t: tag to use if different from version"
     echo "        -r: revision to package"
     echo "        -b: package from branch BRANCH"
     echo "        -T: package from trunk"
     echo "        -d: package from local directory"
+    echo "        -o: owner used for creating tar archive"
+    echo "        -g: group used for creating tar archive"
     echo "        -p: GNU PG passphrrase used for signing"
+    echo "        -k: ID of GNU PG key to use for signing"
 }
 
 copy_files() {
@@ -69,19 +74,27 @@ copy_files() {
 
 #################### MAIN ##############
 
+txtgen=n
 conflict=0
-while getopts :t:r:b:d:p:T c
+while getopts :v:t:r:b:d:p:k:o:g:Tfh c
 do
     case $c in
-    t)         tag=$OPTARG;;
+    v)         version=$OPTARG;;
+    t)         tag=$OPTARG
+               conflict=$(($conflict+1));;
     r)         revision=$OPTARG;;
-    p)         SIGN_OPTS="--passphrase=$OPTARG";;
+    k)         SIGN_OPTS="--default-key=$OPTARG $SIGN_OPTS";;
+    p)         SIGN_OPTS="--passphrase=$OPTARG $SIGN_OPTS";;
+    o)         JK_OWNER=$OPTARG;;
+    g)         JK_GROUP=$OPTARG;;
     b)         branch=$OPTARG
                conflict=$(($conflict+1));;
     T)         trunk=trunk
                conflict=$(($conflict+1));;
     d)         local_dir=$OPTARG
                conflict=$(($conflict+1));;
+    f)         force='y';;
+    h)         txtgen='y';;
     \:)        usage
                exit 2;;
     \?)        usage
@@ -93,7 +106,7 @@ shift `expr $OPTIND - 1`
 if [ $conflict -gt 1 ]
 then
     usage
-    echo "Only one of the options '-b', '-T'  and '-d' is allowed."
+    echo "Only one of the options '-t', '-b', '-T'  and '-d' is allowed."
     exit 2
 fi
 
@@ -114,7 +127,7 @@ then
     fi
 fi
 
-if [ -z "$tag" ]
+if [ -z "$version" ]
 then
     usage
     exit 2
@@ -126,14 +139,21 @@ fi
 if [ -n "$trunk" ]
 then
     JK_SVN_URL="${SVNROOT}/${SVNPROJ}/trunk"
-    JK_REV=`svn info $revision ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
+    svn_url_info="`svn help info | grep URL`"
+    if [ -n "$svn_url_info" ]
+    then
+	JK_SVN_INFO="${JK_SVN_URL}"
+    else
+	JK_SVN_INFO=.
+    fi
+    JK_REV=`svn info $revision $JK_SVN_INFO | awk '$1 == "Revision:" {print $2}'`
     if [ -z "$JK_REV" ]
     then
        echo "No Revision found at '$JK_SVN_URL'"
        exit 3
     fi
-    JK_SUFFIX=${JK_REV}
-    JK_DIST=${JK_CVST}-${tag}-dev-${JK_SUFFIX}-src
+    JK_SUFFIX=-${JK_REV}
+    JK_DIST=${JK_CVST}-${version}-dev${JK_SUFFIX}-src
 elif [ -n "$branch" ]
 then
     JK_BRANCH=`echo $branch | sed -e 's#/#__#g'`
@@ -144,8 +164,8 @@ then
        echo "No Revision found at '$JK_SVN_URL'"
        exit 3
     fi
-    JK_SUFFIX=${JK_BRANCH}-${JK_REV}
-    JK_DIST=${JK_CVST}-${tag}-dev-${JK_SUFFIX}-src
+    JK_SUFFIX=-${JK_BRANCH}-${JK_REV}
+    JK_DIST=${JK_CVST}-${version}-dev${JK_SUFFIX}-src
 elif [ -n "$local_dir" ]
 then
     JK_SVN_URL="$local_dir"
@@ -155,20 +175,39 @@ then
        echo "No Revision found at '$JK_SVN_URL'"
        exit 3
     fi
-    JK_SUFFIX=local-`date +%Y%m%d%H%M%S`-${JK_REV}
-    JK_DIST=${JK_CVST}-${tag}-dev-${JK_SUFFIX}-src
+    JK_SUFFIX=-local-`date +%Y%m%d%H%M%S`-${JK_REV}
+    JK_DIST=${JK_CVST}-${version}-dev${JK_SUFFIX}-src
 else
-    JK_VER=$tag
-    JK_TAG=`echo $tag | sed -e 's#^#JK_#' -e 's#\.#_#g'`
+    JK_VER=$version
+    JK_TAG=`echo $version | sed -e 's#^#JK_#' -e 's#\.#_#g'`
+    if [ -n $tag ]
+    then
+        if [ -z $force ]
+        then
+            echo $tag | grep "^$JK_TAG" > /dev/null 2>&1
+            if [ $? -gt 0 ]
+            then
+                echo "Tag '$tag' doesn't belong to version '$version'."
+                echo "Force using '-f' if you are sure."
+                exit 5
+            fi
+        fi
+        JK_TAG=$tag
+    fi
     JK_SVN_URL="${SVNROOT}/${SVNPROJ}/tags/${JK_TAG}"
-    JK_SUFFIX=''
+    JK_REV=`svn info $revision ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
+    JK_SUFFIX=" ($JK_REV)"
     JK_DIST=${JK_CVST}-${JK_VER}-src
 fi
 
+echo "Using subversion URL: $JK_SVN_URL"
+echo "Rolling into file $JK_DIST.*"
+sleep 2
+
 umask 022
 
-rm -rf ${JK_DIST}
-rm -rf ${JK_DIST}.tmp
+rm -rf ${JK_DIST} 2>/dev/null || true
+rm -rf ${JK_DIST}.* 2>/dev/null || true
 
 mkdir -p ${JK_DIST}.tmp
 svn export $revision "${JK_SVN_URL}" ${JK_DIST}.tmp/jk
@@ -177,25 +216,12 @@ if [ $? -ne 0 ]; then
   exit 1
 fi
 
-for item in ${COPY_TOP}
-do
-    svn export $revision "${JK_SVN_URL}/${item}" ${JK_DIST}.tmp/${item}
-done
-
 # Build documentation.
 cd ${JK_DIST}.tmp/jk/xdocs
 ant
 cd ../../..
 
-# Update version information
-file=${JK_DIST}.tmp/jk/native/common/jk_version.h
-cp -p $file $file.orig
-sed -e 's/^#define JK_REVISION .*/#define JK_REVISION "-'$JK_SUFFIX'"/' \
-  $file.orig > $file
-rm $file.orig
-
 # Copying things into the source distribution
-copy_files ${JK_DIST}.tmp $JK_DIST "$COPY_TOP"
 copy_files ${JK_DIST}.tmp/jk $JK_DIST "$COPY_JK"
 copy_files ${JK_DIST}.tmp/jk/native $JK_DIST "$COPY_NATIVE"
 copy_files ${JK_DIST}.tmp/jk/build $JK_DIST "$COPY_BUILD"
@@ -204,22 +230,24 @@ copy_files ${JK_DIST}.tmp/jk/conf $JK_DIST/conf "$COPY_CONF"
 # Remove extra directories and files
 targetdir=${JK_DIST}
 rm -rf ${targetdir}/xdocs/jk2
-rm -rf ${targetdir}/native/CHANGES.txt
-rm -rf ${targetdir}/native/build.xml
-rm -rf ${targetdir}/native/NOTICE
-rm -rf ${targetdir}/native/LICENSE
+rm -f ${targetdir}/native/build.xml
+rm -f ${targetdir}/native/NOTICE
+rm -f ${targetdir}/native/LICENSE
 find ${JK_DIST} -name .cvsignore -exec rm -rf \{\} \; 
 find ${JK_DIST} -name CVS -exec rm -rf \{\} \; 
 find ${JK_DIST} -name .svn -exec rm -rf \{\} \; 
 
 cd ${JK_DIST}/native
 
+if [ $txtgen = y ]
+then
 # Check for links, elinks or w3m
 W3MOPTS="-dump -cols 80 -t 4 -S -O iso-8859-1 -T text/html"
-ELNKOPTS="-dump -dump-width 80 -dump-charset iso-8859-1 -no-numbering -no-references -no-home"
+ELNKOPTS="-dump -dump-width 80 -dump-charset iso-8859-1 -no-numbering -no-home -no-references"
 LNKOPTS="-dump -width 80 -codepage iso-8859-1 -no-g -html-numbered-links 0"
+LYXOPTS="-dump -width=80 -nolist -nostatus -noprint -assume_local_charset=iso-8859-1"
 failed=true
-for tool in `echo "w3m elinks links"`
+for tool in `echo "w3m elinks links lynx"`
 do
   found=false
   for dir in `echo ${PATH} | sed 's!^:!.:!;s!:$!:.!;s!::!:.:!g;s!:! !g'`
@@ -244,6 +272,9 @@ do
       elinks)
         TOOL="elinks $ELNKOPTS"
         ;;
+      lynx)
+        TOOL="lynx $LYXOPTS"
+        ;;
     esac
     rm -f CHANGES
     echo "Creating the CHANGES file using '$TOOL' ..."
@@ -276,13 +307,14 @@ then
   echo "Can't convert html to text (NEWS)"
   exit 1
 fi
+fi
 
 # Generate configure et. al.
 ./buildconf.sh
 cd ../../
 
 # Pack
-tar cfz ${JK_DIST}.tar.gz --owner="${JK_OWNER}" --group="${JK_GROUP}" ${JK_DIST}
+tar cfz ${JK_DIST}.tar.gz --owner="${JK_OWNER}" --group="${JK_GROUP}" ${JK_DIST} || exit 1
 perl ${JK_DIST}/tools/lineends.pl --cr ${JK_DIST}
 zip -9 -r ${JK_DIST}.zip ${JK_DIST}
 
@@ -291,4 +323,6 @@ archive=${JK_DIST}.tar.gz
 . ${JK_TOOLS}/signfile.sh ${SIGN_OPTS} $archive
 archive=${JK_DIST}.zip
 . ${JK_TOOLS}/signfile.sh ${SIGN_OPTS} $archive
-
+# Cleanup working dirs
+rm -rf ${JK_DIST}.tmp
+rm -rf ${JK_DIST}
